﻿--[[
	Copyright (c) 2009, dr_AllCOM3
    All rights reserved.

    You're allowed to use this addon, free of monetary charge,
    but you are not allowed to modify, alter, or redistribute
    this addon without express, written permission of the author.
]]

local addon = LibStub( "AceAddon-3.0" ):NewAddon( "DocsDebugRunes", "AceEvent-3.0" )
DocsDebugRunes = addon
local L = LibStub( "AceLocale-3.0" ):GetLocale( "DocsDebugRunes" )
local LSM = LibStub("LibSharedMedia-3.0")

local db, temp
local debug = false
local version, build, date, tocversion = GetBuildInfo()

local localization = {
    ff = GetSpellInfo( 59921 ),
    bp = GetSpellInfo( 59879 ),
    km = GetSpellInfo( 51128 ),
    sd = GetSpellInfo( 49530 ),
    dt = GetSpellInfo( 63560 ),
    si = GetSpellInfo( 49572 ),
    rc = GetSpellInfo( 51462 ),
    re = GetSpellInfo( 81229 ),
}

local function p( text ) -- Print
	ChatFrame3:AddMessage( tostring( text ) )
end

local function lightenColor( r, g, b )
    local modifier = 1.5
    
    return r+( 1-r )/modifier, g+( 1-g )/modifier, b+( 1-b )/modifier
end

local function round( num, idp ) --number, decimal places
	local mult = 10^( idp or 0 )
	return math.floor( num * mult + 0.5 ) / mult
end

local function formatTime( time )
    if time>60 then
        time = floor( time/60 ).."m"
    elseif time>=3 then
        time = floor( time )
    else
        time = round( time, 1 )
    end
    
    return time
end

local function createImprovedStatusbar( self, existingBar ) -- Creates a texture that acts like a statusbar. This gets rid of the ugly tiling.
    local function UpdateBar( self )
        local fraction = .01
        local range = self.MaxVal-self.MinVal 
        local value = self.Value-self.MinVal
        local barsize = self.Dim or 100
        
        if range>0 and value>0 and range>=value then
            fraction = value/range
        else
            fraction = .0001
        end
        
        if self.Orientation=="VERTICAL" then 
            self.Bar:SetHeight( barsize*fraction )
            self.Bar:SetTexCoord( 0, 1, 1-fraction, 1 )
        else 
            self.Bar:SetWidth( barsize*fraction ) 
            self.Bar:SetTexCoord( 0, fraction, 0, 1 )
        end
    end
    
    local function UpdateSize( self ) 
        if self.Orientation=="VERTICAL" then
            self.Dim = self:GetHeight()
        else
            self.Dim = self:GetWidth()
        end
        
        UpdateBar( self ) 
    end
    
    local function SetValue( self, value )
        if value>=self.MinVal and value<=self.MaxVal then self.Value = value end
        
        UpdateBar( self ) 
    end
    
    local function GetValue( self )
        return self.Value
    end
    
    local function SetMinMaxValues( self, minval, maxval )
        if maxval>minval then
            self.MinVal = minval
            self.MaxVal = maxval
        else 
            self.MinVal = 0
            self.MaxVal = 100
        end
        
        if self.Value>self.MaxVal then self.Value = self.MaxVal
        elseif self.Value<self.MinVal then self.Value = self.MinVal end
        
        UpdateBar( self ) 
    end
    
    
    local function GetMinMaxValues( self )
        return self.MinVal, self.MaxVal
    end
    
    
    local function SetOrientation( self, orientation ) 
        if orientation=="VERTICAL" then
            self.Orientation = orientation
            self.Bar:ClearAllPoints()
            self.Bar:SetPoint( "BOTTOMLEFT" )
            self.Bar:SetPoint( "BOTTOMRIGHT" )
        else
            self.Orientation = "HORIZONTAL"
            self.Bar:ClearAllPoints()
            self.Bar:SetPoint( "TOPLEFT" )
            self.Bar:SetPoint( "BOTTOMLEFT" )
        end
        
        UpdateSize( self )
    end
    
    local function SetStatusBarColor( self, r, g, b, a )
        self.Bar:SetVertexColor( r, g, b, a )
    end
    
    local function SetStatusBarTexture( self, texture )
        self.Bar:SetTexture( texture )
    end
    
    local statusbar
    if not existingBar then
        statusbar = CreateFrame( "Frame", nil, self )
    else
        statusbar = existingBar
    end
    --statusbar:SetPoint( "TOP", self, "TOP", 0, 0 )
    statusbar:SetWidth( db.width )
    statusbar:SetHeight( db.height )
    
    if not statusbar.Bar then
        temp = statusbar:CreateTexture( nil, "ARTWORK" )
    else
        temp = statusbar.Bar
    end
    statusbar.Bar = temp
    
    statusbar.Value = 100
    statusbar.MinVal = 0
    statusbar.MaxVal = 100
    statusbar.Orientation = "HORIZONTAL"
    statusbar.UpdateSize = UpdateSize
	statusbar.SetValue = SetValue
    statusbar.GetValue = GetValue
	statusbar.SetMinMaxValues = SetMinMaxValues
    statusbar.GetMinMaxValues = GetMinMaxValues
	statusbar.SetOrientation = SetOrientation
	statusbar.SetStatusBarColor = SetStatusBarColor
	statusbar.SetStatusBarTexture = SetStatusBarTexture
    
    --statusbar:SetFrameLevel( level )
    statusbar:SetOrientation( "HORIZONTAL" )
    statusbar:SetStatusBarTexture( LSM:Fetch( "statusbar", db.texture ) )
    --DocsUI_Nameplates.setBorder( statusbar )
	
	statusbar:SetScript( "OnSizeChanged", UpdateSize )
	UpdateSize( statusbar )
    
    return statusbar
end

local runes = {
    {
        { rune=1, ready = true, runeType = 1, duration = 10, start = 0 },    -- RUNETYPE_BLOOD = 1
        { rune=2, ready = true, runeType = 1, duration = 10, start = 0 },    -- RUNETYPE_DEATH = 2
    },
    {
        { rune=1, ready = true, runeType = 2, duration = 10, start = 0 },    -- RUNETYPE_FROST = 3
        { rune=2, ready = true, runeType = 2, duration = 10, start = 0 },    -- RUNETYPE_CHROMATIC = 4
    },
    {
        { rune=1, ready = true, runeType = 3, duration = 10, start = 0 },
        { rune=2, ready = true, runeType = 3, duration = 10, start = 0 },
    },
}

local colors = {
    { 0.77, 0.12, 0.23, 1 }, -- RUNETYPE_BLOOD = 1
    { 0.3, 0.8, 0.1, 1 }, -- RUNETYPE_DEATH = 2
    { 0, 0.4, 0.7, 1 }, -- RUNETYPE_FROST = 3
    { 0.51, 0.23, 0.65, 1 }, -- RUNETYPE_CHROMATIC = 4
}

local colorsCooldown = {
    { 1, 0, 0, 0.33 }, -- RUNETYPE_BLOOD = 1
    { 0, 1, 0, 0.33 }, -- RUNETYPE_DEATH = 2
    { 0, 0, 1, 0.33 }, -- RUNETYPE_FROST = 3
    { 1, 0, 1, 0.33 }, -- RUNETYPE_CHROMATIC = 4
}

-- onUpdate
local function onUpdate( self, elapsed )
    local time = GetTime()
    
    if ( ( db.enableDiseases and self.type=="disease" ) or ( db.enableDarkTransformation and self.type=="pet" ) ) and self.timer then -- Diseases or pet
        self:SetValue( self.expirationTime-time )
        if self.count then
            self.timer:SetText( self.count )
        else
            self.timer:SetText( formatTime( round( self.expirationTime-time, 1 ) ) )
        end
        
        if debug then
            self.text:SetText( formatTime( round( self.expirationTime-time, 1 ) ) .." | ".. formatTime( round( self.duration, 1 ) ) )
        end
        
        if self.expirationTime-time<=0 then
            self.timer:SetText( nil )
            self.text:SetText( nil )
            self:Hide()
        end
    else -- Runes
        local duration = self.duration+elapsed
        self.lastDuration = duration
        
        if duration>=self.max then
            self.text:SetText( nil )
            self:SetScript( "OnUpdate", nil )
        else
            if debug then
                self.text:SetText( formatTime( round( self.duration, 1 ) ) .." | ".. formatTime( round( self.max, 1 ) ) )
            end
            self.duration = duration
            self:SetValue( duration )
        end
    end
end

local frames = {}
local function createFrames()
    temp = CreateFrame( "Frame", "DocsDebugRunesFrame", UIParent )
    temp:SetWidth( db.width*2+db.spacing*2+db.spacing*3 )
    temp:SetHeight( db.height*3+db.spacing*3+db.spacing*3 )
    temp:SetBackdrop( { 
      bgFile = "Interface\\Buttons\\WHITE8X8", 
      edgeFile = LSM:Fetch( "border", db.border ), tile = false, tileSize = 16, edgeSize = 16, 
      insets = { left = 4, right = 4, top = 4, bottom = 4 }
    } )
    temp:SetBackdropBorderColor( 0.5, 0.5, 0.5, 1 )
    temp:SetBackdropColor( 0.1, 0.1, 0.1, 1 )
    temp:EnableMouse( true )
    temp:SetClampedToScreen( true )
    temp:SetMovable( true )
    --temp:SetPoint( DocsDebugRunesDB.point or "CENTER", UIParent, DocsDebugRunesDB.point or "CENTER", DocsDebugRunesDB.xOfs or 0, DocsDebugRunesDB.yOfs or 0 )
    temp:SetScript( "OnMouseDown", function( self )
            if IsShiftKeyDown() then
                self:StartMoving()
                frames.tutorialMove:Show()
            end
        end)
    temp:SetScript( "OnMouseUp", function( self, button )
            if button=="RightButton" then
                debug = not debug
                ChatFrame1:AddMessage( "DocsDebugRunes Debug: "..tostring( debug ) )
            else
                self:StopMovingOrSizing()
                
                local point, relativeTo, relativePoint, xOfs, yOfs = self:GetPoint( 1 )
                db.point = point
                db.xOfs = xOfs
                db.yOfs = yOfs
                
                db.tutorialMove = true
                frames.tutorialMove:Hide()
            end
        end )
    frames.bg = temp
    
    for i=1,6 do
        frames[i] = createImprovedStatusbar( frames.bg, frames[i] )
        frames[i]:SetWidth( db.width )
        frames[i]:SetHeight( db.height )
        frames[i]:SetStatusBarColor( 1, 1, 1, 1 )
        frames[i]:SetMinMaxValues( 0, 1 )
        frames[i]:SetValue( 1 )
        frames[i]:UpdateSize( frames[i] )
        
        frames[i].bg = frames[i]:CreateTexture( nil, "BACKGROUND" )
        frames[i].bg:SetAllPoints( frames[i] )
        frames[i].bg:SetTexture( db.texture )
        
        frames[i].text = frames.bg:CreateFontString( nil, "OVERLAY", "GameFontNormal" )
        --frames[i].text:SetWidth( db.width )
        --frames[i].text:SetHeight( db.height )
        frames[i].text:SetFont( LSM:Fetch( "font", db.font ), db.fontsize, "THINOUTLINE" )
        frames[i].text:SetJustifyH( "CENTER" )
        frames[i].text:SetJustifyV( "CENTER" )
        frames[i].text:SetTextColor( 0.66, 0.66, 0.66, 1 )
        if i==1 or i==3 or i==5 then
            frames[i].text:SetPoint( "RIGHT", frames[i], "LEFT", -db.spacing*2, 0 )
        else
            frames[i].text:SetPoint( "LEFT", frames[i], "RIGHT", db.spacing*2, 0 )
        end
        --frames[i].text:SetText( "test" )
        
        frames[i].glow = CreateFrame( "Frame", nil, frames[i] )
        frames[i].glow:SetAllPoints( frames[i] )
        frames[i].glow:Hide()
        frames[i].glow:SetScript( "OnUpdate", function( self )
            self.texture:SetAlpha( self.alpha )
            self.alpha = self.alpha-self.alpha*0.03
            if self.alpha<=0.1 then
                self:Hide()
            end
        end )
        frames[i].glow:SetScript( "OnShow", function( self )
            self.texture:SetAlpha( 1 )
            self.alpha = 1
        end )
        
        frames[i].glow.texture = frames[i].glow:CreateTexture( nil, "OVERLAY" )
        frames[i].glow.texture:SetAllPoints( frames[i].glow )
        frames[i].glow.texture:SetTexture( "Interface\\Buttons\\WHITE8X8" )
        
        frames[i].start = 0
        frames[i].max = 0
        frames[i].duration = 0
        frames[i].lastDuration = 0
        frames[i].switched = false
        frames[i].lastUpdate = 0
        frames[i].onUpdate = onUpdate
    end
    
    frames.glow = CreateFrame( "Frame", nil, frames.bg )
    local inset = 2
    frames.glow:SetPoint( "TOPLEFT", frames.bg, "TOPLEFT", inset, -inset )
    frames.glow:SetPoint( "BOTTOMRIGHT", frames.bg, "BOTTOMRIGHT", -inset, inset )
    frames.glow:Hide()
    frames.glow.visible = false
    
    temp = frames.glow:CreateTexture( nil, "OVERLAY" )
    temp:SetAllPoints( frames.glow )
    temp:SetTexture( "Interface\\Addons\\DocsDebugRunes\\rcGlow" )
    temp:SetVertexColor( 1,1,1,0.6 )
    
    
    -- Rune order
    do
    frames[5]:SetPoint( "TOPLEFT", frames.bg, "TOPLEFT", db.spacing*2, -db.spacing*2 )
    frames[5]:SetStatusBarColor( unpack(colors[3] ) )
    frames[6]:SetPoint( "LEFT", frames[5], "RIGHT", db.spacing, 0 )
    frames[6]:SetStatusBarColor( unpack(colors[3] ) )
    frames[3]:SetPoint( "TOP", frames[5], "BOTTOM", 0, -db.spacing )
    frames[3]:SetStatusBarColor( unpack(colors[2] ) )
    frames[4]:SetPoint( "LEFT", frames[3], "RIGHT", db.spacing, 0 )
    frames[4]:SetStatusBarColor( unpack(colors[2] ) )
    frames[1]:SetPoint( "TOP", frames[3], "BOTTOM", 0, -db.spacing )
    frames[1]:SetStatusBarColor( unpack(colors[1] ) )
    frames[2]:SetPoint( "LEFT", frames[1], "RIGHT", db.spacing, 0 )
    frames[2]:SetStatusBarColor( unpack(colors[1] ) )
    end
    
    -- Runic power
    do
    frames.rp = createImprovedStatusbar( frames.bg, frames.rp )
    frames.rp:SetWidth( db.width*2+db.spacing*1 )
    frames.rp:SetHeight( db.height )
    frames.rp:UpdateSize( frames.rp )
    frames.rp:SetStatusBarColor( 0, 0.82, 1, 1 )
    frames.rp:SetPoint( "TOP", frames.bg, "BOTTOM", 0, 0 )
    frames.rp:SetMinMaxValues( 0, UnitManaMax( "player" ) )
    if not db.enableRunicPower then frames.rp:Hide() end
    
    frames.rp.text = frames.rp:CreateFontString( nil, "OVERLAY", "GameFontNormal" )
    --frames.rp.text:SetWidth( db.width )
    --frames.rp.text:SetHeight( db.height )
    frames.rp.text:SetFont( LSM:Fetch( "font", db.font ), db.fontsize+2, "THINOUTLINE" )
    frames.rp.text:SetJustifyH( "CENTER" )
    frames.rp.text:SetJustifyV( "CENTER" )
    frames.rp.text:SetTextColor( 0, 0.82, 1, 1 )
    frames.rp.text:SetPoint( "CENTER", frames.rp, "CENTER", 0, 0 )
    
    frames.rp.bg = frames.rp:CreateTexture( nil, "BACKGROUND" )
    frames.rp.bg:SetTexture( "Interface\\Buttons\\WHITE8X8" )
    frames.rp.bg:SetAllPoints( frames.rp )
    frames.rp.bg:SetVertexColor( 0, 0.205, 0.25, 1 )
    end
    
    -- Diseases
    do
    frames.bp = createImprovedStatusbar( frames.bg, frames.bp )
    frames.bp:SetWidth( db.width )
    frames.bp:SetHeight( db.height )
    frames.bp:UpdateSize( frames.bp )
    frames.bp:SetStatusBarColor( 0.5, 0, 0.6, 1 )
    frames.bp:SetPoint( "BOTTOMLEFT", frames.bg, "TOPLEFT", db.spacing*2, 0 )
    frames.bp.type = "disease"
    frames.bp.lastUpdate = 0
    frames.bp:SetScript( "OnUpdate", onUpdate )
    frames.bp:Hide()
    
    frames.bp.timer = frames.bp:CreateFontString( nil, "OVERLAY", "GameFontNormal" )
    frames.bp.timer:SetFont( LSM:Fetch( "font", db.font ), db.fontsize+2, "THINOUTLINE" )
    frames.bp.timer:SetJustifyH( "CENTER" )
    frames.bp.timer:SetJustifyV( "CENTER" )
    frames.bp.timer:SetTextColor( lightenColor( 0.5, 0, 0.6 ) )
    frames.bp.timer:SetPoint( "CENTER", frames.bp, "TOP", 0, 0 )
    
    frames.bp.text = frames.bp:CreateFontString( nil, "OVERLAY", "GameFontNormal" )
    frames.bp.text:SetFont( LSM:Fetch( "font", db.font ), db.fontsize, "THINOUTLINE" )
    frames.bp.text:SetJustifyH( "CENTER" )
    frames.bp.text:SetJustifyV( "CENTER" )
    frames.bp.text:SetTextColor( 0.66, 0.66, 0.66, 1 )
    frames.bp.text:SetPoint( "RIGHT", frames.bp, "LEFT", -db.spacing*2, 0 )
    
    frames.bp.bg = frames.bp:CreateTexture( nil, "BACKGROUND" )
    frames.bp.bg:SetTexture( "Interface\\Buttons\\WHITE8X8" )
    frames.bp.bg:SetAllPoints( frames.bp )
    frames.bp.bg:SetVertexColor( 0.125, 0, 0.15, 1 )
    
    frames.ff = createImprovedStatusbar( frames.bg, frames.ff )
    frames.ff:SetWidth( db.width )
    frames.ff:SetHeight( db.height )
    frames.ff:UpdateSize( frames.ff )
    frames.ff:SetStatusBarColor( 0, 0.45, 0.6, 1 )
    frames.ff:SetPoint( "BOTTOMRIGHT", frames.bg, "TOPRIGHT", -db.spacing*2, 0 )
    frames.ff.type = "disease"
    frames.ff.lastUpdate = 0
    frames.ff:SetScript( "OnUpdate", onUpdate )
    frames.ff:Hide()
    
    frames.ff.timer = frames.ff:CreateFontString( nil, "OVERLAY", "GameFontNormal" )
    frames.ff.timer:SetFont( LSM:Fetch( "font", db.font ), db.fontsize+2, "THINOUTLINE" )
    frames.ff.timer:SetJustifyH( "CENTER" )
    frames.ff.timer:SetJustifyV( "CENTER" )
    frames.ff.timer:SetTextColor( lightenColor( 0, 0.45, 0.6 ) )
    frames.ff.timer:SetPoint( "CENTER", frames.ff, "TOP", 0, 0 )
    
    frames.ff.text = frames.ff:CreateFontString( nil, "OVERLAY", "GameFontNormal" )
    frames.ff.text:SetFont( LSM:Fetch( "font", db.font ), db.fontsize, "THINOUTLINE" )
    frames.ff.text:SetJustifyH( "CENTER" )
    frames.ff.text:SetJustifyV( "CENTER" )
    frames.ff.text:SetTextColor( 0.66, 0.66, 0.66, 1 )
    frames.ff.text:SetPoint( "LEFT", frames.ff, "RIGHT", db.spacing*2, 0 )
    
    frames.ff.bg = frames.ff:CreateTexture( nil, "BACKGROUND" )
    frames.ff.bg:SetTexture( "Interface\\Buttons\\WHITE8X8" )
    frames.ff.bg:SetAllPoints( frames.ff )
    frames.ff.bg:SetVertexColor( 0, 0.1125, 0.15, 1 )
    end
    
    -- Pet
    do
    frames.dt = createImprovedStatusbar( frames.bg, frames.dt )
    frames.dt:SetWidth( db.width )
    frames.dt:SetHeight( db.height )
    frames.dt:UpdateSize( frames.dt )
    frames.dt:SetStatusBarColor( 0, 0.5, 0, 1 )
    frames.dt:SetPoint( "BOTTOM", frames.bg, "TOP", 0, db.spacing*5 )
    frames.dt.type = "pet"
    frames.dt.lastUpdate = 0
    frames.dt:SetScript( "OnUpdate", onUpdate )
    frames.dt:Hide()
    
    frames.dt.timer = frames.dt:CreateFontString( nil, "OVERLAY", "GameFontNormal" )
    frames.dt.timer:SetFont( LSM:Fetch( "font", db.font ), db.fontsize+2, "THINOUTLINE" )
    frames.dt.timer:SetJustifyH( "CENTER" )
    frames.dt.timer:SetJustifyV( "CENTER" )
    frames.dt.timer:SetTextColor( lightenColor( 0.5, 0, 0.6 ) )
    frames.dt.timer:SetPoint( "CENTER", frames.dt, "TOP", 0, 0 )
    
    frames.dt.text = frames.dt:CreateFontString( nil, "OVERLAY", "GameFontNormal" )
    --frames.dt.text:SetWidth( db.width )
    --frames.dt.text:SetHeight( db.height )
    frames.dt.text:SetFont( LSM:Fetch( "font", db.font ), db.fontsize, "THINOUTLINE" )
    frames.dt.text:SetJustifyH( "CENTER" )
    frames.dt.text:SetJustifyV( "CENTER" )
    frames.dt.text:SetTextColor( 0.66, 0.66, 0.66, 1 )
    frames.dt.text:SetPoint( "RIGHT", frames.dt, "LEFT", -db.spacing*2, 0 )
    
    frames.dt.bg = frames.dt:CreateTexture( nil, "BACKGROUND" )
    frames.dt.bg:SetTexture( "Interface\\Buttons\\WHITE8X8" )
    frames.dt.bg:SetAllPoints( frames.dt )
    frames.dt.bg:SetVertexColor( 0.125, 0, 0.15, 1 )
    end
    
    -- Procs
    do
    frames.proc = frames.bg:CreateTexture( nil, "ARTWORK" )
    frames.proc:SetWidth( db.width )
    frames.proc:SetHeight( db.width )
    frames.proc:SetPoint( "BOTTOM", frames.dt, "TOP", 0, db.spacing*2 )
    frames.proc:SetVertexColor( 1,1,1,1 )
    frames.proc:Hide()
    end
    
    -- Tutorial
    do
    frames.tutorialMove = frames.bg:CreateFontString( nil, "OVERLAY", "GameFontNormal" )
    --frames.tutorialMove:SetWidth( db.width )
    --frames.tutorialMove:SetHeight( db.height )
    frames.tutorialMove:SetFont( LSM:Fetch( "font", db.font ), db.fontsize+4, "THINOUTLINE" )
    frames.tutorialMove:SetJustifyH( "CENTER" )
    frames.tutorialMove:SetJustifyV( "CENTER" )
    frames.tutorialMove:SetTextColor( 1, 1, 1, 1 )
    frames.tutorialMove:SetPoint( "LEFT", frames.bg, "RIGHT", 0, 0 )
    frames.tutorialMove:SetText( "You can move me with Shift+Drag" )
    end
    
end

-- Update diseases
local debuffs = { target = {}, player = {}, pet = {} }
local function aura( unit )
    local counter = 1
    
    if unit=="target" then
        wipe( debuffs[unit] )
        
        for i=1,40 do
            local name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId = UnitAura( unit, i, "HARMFUL" )
            
            if name and unitCaster=="player" and name==localization.ff then
                debuffs[unit].ff = { spellId, name, icon, count, debuffType, duration, expirationTime, isMine }
            elseif name and unitCaster=="player" and name==localization.bp then
                debuffs[unit].bp = { spellId, name, icon, count, debuffType, duration, expirationTime, isMine }
            elseif not name then
                break
            end
        end
        
        if db.enableDiseases and debuffs[unit].ff then --"Frost Fever"
            local time = GetTime()
            
            frames.ff:SetMinMaxValues( 0, debuffs[unit].ff[6] )
            frames.ff:SetValue( time-( debuffs[unit].ff[7]-time ) )
            
            frames.ff.expirationTime = debuffs[unit].ff[7]
            frames.ff.duration = debuffs[unit].ff[6]
            frames.ff:SetMinMaxValues( 0, debuffs[unit].ff[6] )
            frames.ff:Show()
        else
            frames.ff:Hide()
        end
        
        if db.enableDiseases and debuffs[unit].bp then --"Blood Plague"
            local time = GetTime()
            
            frames.bp:SetMinMaxValues( 0, debuffs[unit].bp[6] )
            frames.bp:SetValue( time-( debuffs[unit].bp[7]-time ) )
            
            frames.bp.expirationTime = debuffs[unit].bp[7]
            frames.bp.duration = debuffs[unit].bp[6]
            frames.bp:SetMinMaxValues( 0, debuffs[unit].bp[6] )
            frames.bp:Show()
        else
            frames.bp:Hide()
        end
    elseif unit=="player" then
        wipe( debuffs[unit] )
        
        for i=1,40 do
            local name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId = UnitAura( unit, i, "HELPFUL" )
            
            if name and unitCaster=="player" and name==localization.km then
                debuffs[unit].km = { spellId, name, icon, count, debuffType, duration, expirationTime, isMine }
            elseif name and unitCaster=="player" and name==localization.sd then
                debuffs[unit].sd = { spellId, name, icon, count, debuffType, duration, expirationTime, isMine }
            elseif name and unitCaster=="player" and name==localization.rc then
                debuffs[unit].rc = { spellId, name, icon, count, debuffType, duration, expirationTime, isMine }
            elseif not name then
                break
            end
        end
        
        if db.enableProcs and debuffs[unit].km then --"Killing Machine"
            frames.proc:SetTexture( debuffs[unit].km[3] )
            frames.proc:Show()
        elseif db.enableProcs and debuffs[unit].sd then --"Sudden Doom"
            frames.proc:SetTexture( debuffs[unit].sd[3] )
            frames.proc:Show()
        else
            frames.proc:Hide()
        end
        
        if debuffs[unit].rc then
            if not frames.glow.visible then
                frames.glow:Show()
                frames.glow.visible = true
            end
        else
            if frames.glow.visible then
                frames.glow:Hide()
                frames.glow.visible = false
            end
        end
    elseif unit=="pet" then
        wipe( debuffs[unit] )
        
        for i=1,40 do
            local name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId = UnitAura( unit, i, "HELPFUL" )
            
            if name and name==localization.dt then
                debuffs[unit].dt = { spellId, name, icon, count, debuffType, duration, expirationTime, isMine }
            elseif name and name==localization.si then
                debuffs[unit].si = { spellId, name, icon, count, debuffType, duration, expirationTime, isMine }
            elseif not name then
                break
            end
        end
        
        if db.enableDarkTransformation and debuffs[unit].dt then
            local time = GetTime()
            
            frames.dt:SetMinMaxValues( 0, debuffs[unit].dt[6] )
            frames.dt:SetValue( time-( debuffs[unit].dt[7]-time ) )
            
            frames.dt.count = nil
            frames.dt.expirationTime = debuffs[unit].dt[7]
            frames.dt.duration = debuffs[unit].dt[6]
            frames.dt:SetMinMaxValues( 0, debuffs[unit].dt[6] )
            
            frames.dt:SetStatusBarColor( 0.5, 0.25, 0, 1 )
            frames.dt.timer:SetTextColor( lightenColor( 0.5, 0.25, 0, 1 ) )
            frames.dt:Show()
        elseif db.enableDarkTransformation and debuffs[unit].si then
            local time = GetTime()
            
            frames.dt:SetMinMaxValues( 0, debuffs[unit].si[6] )
            frames.dt:SetValue( time-( debuffs[unit].si[7]-time ) )
            
            frames.dt.count = debuffs[unit].si[4]
            frames.dt.expirationTime = debuffs[unit].si[7]
            frames.dt.duration = debuffs[unit].si[6]
            frames.dt:SetMinMaxValues( 0, debuffs[unit].si[6] )
            
            frames.dt:SetStatusBarColor( 0, 0.5, 0, 1 )
            frames.dt.timer:SetTextColor( lightenColor( 0, 0.5, 0, 1 ) )
            frames.dt:Show()
        else
            frames.dt:Hide()
        end
    end
end

-- On Rune Update
local function onRuneCooldownUpdate( self, rune )
    if not rune or rune>6 then
        return
    end
    
    local place, pool, modificator
    if rune==1 then
        pool = 1
        place = 1
    elseif rune==2 then
        pool = 1
        place = 2
    elseif rune==3 then
        pool = 2
        place = 1
    elseif rune==4 then
        pool = 2
        place = 2
    elseif rune==5 then
        pool = 3
        place = 1
    elseif rune==6 then
        pool = 3
        place = 2
    end
    if place==1 then modificator = 1 else modificator = -1 end
    
    local runeOther = rune+modificator
    
    local time = GetTime()
    
    local start, duration, runeReady = GetRuneCooldown( rune )
    runes[pool][place].ready = runeReady
    runes[pool][place].duration = time-start
    runes[pool][place].start = start
    runes[pool][place].max = duration
    
    local start, duration, runeReady = GetRuneCooldown( runeOther )
    runes[pool][place+modificator].ready = runeReady
    runes[pool][place+modificator].duration = time-start
    runes[pool][place+modificator].start = start
    runes[pool][place+modificator].max = duration
    
    if time-runes[pool][1].start<time-runes[pool][2].start then
        rune, runeOther = runeOther, rune
    end
    
    if runes[pool][place].ready then
        frames[rune]:SetMinMaxValues( 0, 1 )
        frames[rune]:SetValue( 1 )
        frames[rune]:SetScript( "OnUpdate", nil )
        frames[rune].text:SetText( nil )
        frames[rune]:SetAlpha( 1 )
        if frames[rune].lastDuration>frames[rune].max+0.5 then
            frames[rune].glow:Show()
        end
        frames[rune].lastDuration = 0
    else
        frames[rune].duration = time-runes[pool][place].start
        frames[rune].max = runes[pool][place].max
        frames[rune].start = runes[pool][place].start
        frames[rune]:SetMinMaxValues( 0, frames[rune].max )
        frames[rune]:SetScript( "OnUpdate", frames[rune].onUpdate )
        frames[rune]:SetAlpha( 0.4 )
        
        if frames[rune].duration<0 then frames[rune]:SetValue( 0 ) end
    end
    frames[rune]:SetStatusBarColor( unpack( colors[runes[pool][place].runeType] ) )
    frames[rune].bg:SetVertexColor( unpack( colorsCooldown[runes[pool][place].runeType] ) )
    
    if runes[pool][place+modificator].ready then
        frames[runeOther]:SetMinMaxValues( 0, 1 )
        frames[runeOther]:SetValue( 1 )
        frames[runeOther]:SetScript( "OnUpdate", nil )
        frames[runeOther].text:SetText( nil )
        frames[runeOther]:SetAlpha( 1 )
        if frames[runeOther].lastDuration>frames[runeOther].max+0.5 then
            frames[runeOther].glow:Show()
        end
        frames[runeOther].lastDuration = 0
    else
        frames[runeOther].duration = time-runes[pool][place+modificator].start
        frames[runeOther].max = runes[pool][place+modificator].max
        frames[runeOther].start = runes[pool][place+modificator].start
        frames[runeOther]:SetMinMaxValues( 0, frames[runeOther].max )
        frames[runeOther]:SetScript( "OnUpdate", frames[runeOther].onUpdate )
        frames[runeOther]:SetAlpha( 0.4 )
        
        if frames[runeOther].duration<0 then frames[runeOther]:SetValue( 0 ) end
    end
    frames[runeOther]:SetStatusBarColor( unpack( colors[runes[pool][place+modificator].runeType] ) )
    frames[runeOther].bg:SetVertexColor( unpack( colorsCooldown[runes[pool][place+modificator].runeType] ) )
end

-- Events
function addon:UNIT_POWER( event, unit, type )
    if not unit then unit = "player" end
    
    if db.enableRunicPower and unit=="player" then
        local power = UnitMana( unit )
        
        frames.rp:SetValue( power )
        frames.rp.text:SetText( power )
    end
end

function addon:UNIT_RUNIC_POWER( event, unit )
    if not unit then unit = "player" end
    
    if db.enableRunicPower and unit=="player" then
        local power = UnitMana( unit )
        
        frames.rp:SetValue( power )
        frames.rp.text:SetText( power )
    end
end

function addon:RUNE_POWER_UPDATE( event, rune )
    onRuneCooldownUpdate( self, rune )
end

function addon:RUNE_TYPE_UPDATE( event, rune )
    if not rune or rune>6 then return end
    
    local runeType = GetRuneType( rune )
    
    local place, pool, modificator
    if rune==1 then
        pool = 1
        place = 1
    elseif rune==2 then
        pool = 1
        place = 2
    elseif rune==3 then
        pool = 2
        place = 1
    elseif rune==4 then
        pool = 2
        place = 2
    elseif rune==5 then
        pool = 3
        place = 1
    elseif rune==6 then
        pool = 3
        place = 2
    end
    if place==1 then modificator = 1 else modificator = -1 end
    
    runes[pool][place].runeType = runeType
    
    local runeOther = rune+modificator
    local time = GetTime()
    if time-runes[pool][1].start<time-runes[pool][2].start then
        rune, runeOther = runeOther, rune
    end
    
    frames[rune]:SetStatusBarColor( unpack( colors[runes[pool][place].runeType] ) )
    frames[rune].bg:SetVertexColor( unpack( colorsCooldown[runes[pool][place].runeType] ) )
    frames[runeOther]:SetStatusBarColor( unpack( colors[runes[pool][place+modificator].runeType] ) )
    frames[runeOther].bg:SetVertexColor( unpack( colorsCooldown[runes[pool][place+modificator].runeType] ) )
end

function addon:UNIT_AURA( event, unit )
    if unit=="target" or unit=="player" or unit=="pet" then aura( unit ) end
end

function addon:PLAYER_TARGET_CHANGED( event )
    aura( "target" )
end

function addon:PLAYER_ENTERING_WORLD( event )
    for i=1,6 do
        addon:RUNE_TYPE_UPDATE( nil, i )
    end
    for i=1,6 do
        addon:RUNE_POWER_UPDATE( nil, i )
    end
end

addon.defaults = { profile = {
    tutorialMove = false,
    
    width = 30,
    height = 10,
    spacing = 4,
    fontsize = 12,
    texture = "DocsDebugRunes", -- Standard: "Interface\\TargetingFrame\\UI-StatusBar"
    font = "Segoe UI", -- Standard: "Fonts\\FRIZQT__.TTF"
    border = "Caith", -- Standard: "Interface\\DialogFrame\\UI-DialogBox-Border"
    enableDiseases = true,
    enableDarkTransformation = true,
    enableRunicPower = true,
    enableProcs = true,
} }

-- OnInitialize
function addon:OnInitialize()
    if select( 2, UnitClass( "player" ) )~="DEATHKNIGHT" then return end
    
    -- DB
	self.db = LibStub( "AceDB-3.0" ):New( "DocsDebugRunesDB", self.defaults )
    db = addon.db.profile
    
    -- Options
    self:SpawnOptions()
    LibStub( "AceConfig-3.0" ):RegisterOptionsTable( "DocsDebugRune", addon.options )
    LibStub( "AceConfigDialog-3.0" ):AddToBlizOptions( "DocsDebugRune", "DocsDebugRune" )
    
    -- Profile options
    local AceDBOptions = LibStub( "AceDBOptions-3.0", true )
	if AceDBOptions then
		self.options.args.profiles = AceDBOptions:GetOptionsTable( self.db )
		self.options.args.profiles.order = 999
	end
    
    LSM:Register("font", "Segoe UI",            [[Interface\Addons\DocsDebugRunes\font.ttf]])
    LSM:Register("statusbar", "DocsDebugRunes", [[Interface\Addons\DocsDebugRunes\statusbar]])
    LSM:Register("border", "Caith",             [[Interface\Addons\DocsDebugRunes\caith]])
end
-- OnEnable
function addon:OnEnable()
    if select( 2, UnitClass( "player" ) )~="DEATHKNIGHT" then return end
    
    RuneFrame:Hide()
    
    createFrames()
    frames.bg:ClearAllPoints()
    frames.bg:SetPoint( db.point or "CENTER", UIParent, db.point or "CENTER", db.xOfs or 0, db.yOfs or 0 )
    
    addon:RegisterEvent( "RUNE_POWER_UPDATE" )
    addon:RegisterEvent( "RUNE_TYPE_UPDATE" )
    addon:RegisterEvent( "UNIT_AURA" )
    addon:RegisterEvent( "PLAYER_TARGET_CHANGED" )
    addon:RegisterEvent( "PLAYER_ENTERING_WORLD" )
    if tocversion<40000 then
        addon:RegisterEvent( "UNIT_RUNIC_POWER" )
        addon:UNIT_RUNIC_POWER()
    else
        addon:RegisterEvent( "UNIT_POWER" )
        addon:UNIT_POWER()
    end
    
    if not db.tutorialMove then
        db.tutorialMove = true
    else
        frames.tutorialMove:Hide()
    end
end
function addon:OnDisable()
	
end
