﻿--[[
	Copyright (c) 2009, dr_AllCOM3
    All rights reserved.

    You're allowed to use this addon, free of monetary charge,
    but you are not allowed to modify, alter, or redistribute
    this addon without express, written permission of the author.
]]

DocsUI_Automation = LibStub("AceAddon-3.0"):NewAddon("DocsUI_Automation", "AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("DocsUI")
local LSM = LibStub("LibSharedMedia-3.0")
local db
local temp,temp2 = nil

local _, class = UnitClass("player")

--[[ OnInitialize ]]
function DocsUI_Automation:OnInitialize()
    
end
--[[ OnEnable ]]
function DocsUI_Automation:OnEnable()
    db = DocsUI.db.profile
    
    self:RegisterEvent("MERCHANT_SHOW")
    self:RegisterEvent("UI_ERROR_MESSAGE")
    self:RegisterEvent("TRAINER_SHOW")
    self:RegisterEvent("DUEL_REQUESTED")
    self:RegisterEvent("PARTY_INVITE_REQUEST")
    self:RegisterEvent("RESURRECT_REQUEST")
    self:RegisterEvent("AUCTION_HOUSE_SHOW")
    self:RegisterEvent("PLAYER_REGEN_DISABLED")
    
    --[[ Sort the guild screen ]]
    SortGuildRoster("name")
    SortGuildRoster("class")
end
--[[ OnDisable ]]
function DocsUI_Automation:OnDisable()
	
end

-- MERCHANT_SHOW
-- Sell junk
-- Repair
function DocsUI_Automation:MERCHANT_SHOW()
    for bag=0,4 do
		for slot=0,GetContainerNumSlots(bag) do
			local link = GetContainerItemLink(bag, slot)
			if link and select(3, GetItemInfo(link)) == 0 then
				ShowMerchantSellCursor(1)
				UseContainerItem(bag, slot)
			end
		end
	end
    
    if CanMerchantRepair() and GetRepairAllCost()>0 then 
        RepairAllItems(CanGuildBankRepair() and 1 or 0)
        
        local cost = GetRepairAllCost()
		local gold = floor(cost / 10000)
		local silver = floor((cost - (gold * 10000)) / 100)
		local copper = mod(cost, 100)
		local formattedCost = format("%i|cffffd700%s|r %i|cffc7c7cf%s|r %i|cffeda55f%s|r", gold, "g", silver, "s", copper, "c")
        
        DEFAULT_CHAT_FRAME:AddMessage(L["Repaired all equipment for "]..formattedCost..".")
    end  
end

-- UI_ERROR_MESSAGE
-- Dismount
local messages =    {
                    [ERR_TAXIPLAYERALREADYMOUNTED] = true,
                    [ERR_ATTACK_MOUNTED] = true,
                    [SPELL_FAILED_NOT_MOUNTED] = true,
                    [SPELL_FAILED_OUT_OF_RANGE] = true,
                    [ERR_OUT_OF_RANGE] = true,
                    [ERR_PET_SPELL_OUT_OF_RANGE] = true,
                    [ERR_NOT_WHILE_MOUNTED] = true,
                    }
function DocsUI_Automation:UI_ERROR_MESSAGE()
    if UnitOnTaxi("player") then return end
	if not messages[msg] then return end

	if IsMounted() and not IsFlying() then
		Dismount()
	end
end

-- TRAINER_SHOW
-- Show only available skills at trainer
function DocsUI_Automation:TRAINER_SHOW()
	SetTrainerServiceTypeFilter("unavailable", 0)
end

-- DUEL_REQUESTED
-- Decline duels
function DocsUI_Automation:DUEL_REQUESTED()
    if db.declineDuels then
        DEFAULT_CHAT_FRAME:AddMessage(L["DocsUI: Canceling duel..."])
        CancelDuel()
        StaticPopup_Hide("DUEL_REQUESTED")
    end
end

-- PARTY_INVITE_REQUEST
-- Autoaccept invites
function DocsUI_Automation:PARTY_MEMBERS_CHANGED()
	StaticPopup_Hide("PARTY_INVITE")
	self:UnregisterEvent("PARTY_MEMBERS_CHANGED")
end
function DocsUI_Automation:PARTY_INVITE_REQUEST(self, from)
	local acceptFrom = {}
	GuildRoster()
	
	for i=1,GetNumGuildMembers() do
		local name = GetGuildRosterInfo(i)
		tinsert(acceptFrom,name)
	end
    
	for i=1,GetNumFriends() do
		local name = GetFriendInfo(i)
		tinsert(acceptFrom,name)
	end
    
	if foreachi(acceptFrom, function(i,v) if v==from then return true end end) then
		AcceptGroup()
        DocsUI_Automation:RegisterEvent("PARTY_MEMBERS_CHANGED")
		return
	end
	
	SendWho("n-\""..from.."\"")
end

-- RESURRECT_REQUEST
-- Autoaccept rezz
function DocsUI_Automation:RESURRECT_REQUEST(self, rezzer)
    if GetCorpseRecoveryDelay()>0 or UnitAffectingCombat("player") or UnitAffectingCombat(rezzer) then
        return
    else
        AcceptResurrect()
		StaticPopup_Hide("RESURRECT_NO_SICKNESS")
    end
end

-- AUCTION_HOUSE_SHOW
-- Move auction frame
function DocsUI_Automation:AUCTION_HOUSE_SHOW(self)
    AuctionFrame:ClearAllPoints()
    AuctionFrame:SetPoint("TOPLEFT", -10, 10)
    AuctionFrame:SetScale( 1 )
end

-- PLAYER_REGEN_DISABLED
-- Presence warner
function DocsUI_Automation:PLAYER_REGEN_DISABLED(self)
--~     if class=="DEATHKNIGHT" and db.presenceWarner and UnitInParty("player") and UnitInRaid("player") and not GetBattlefieldScore(1) and not IsActiveBattlefieldArena() then
--~         if select(5, GetTalentInfo(1,3))==0 and select(5, GetTalentInfo(3,3))==0 and not UnitBuff("player", GetSpellInfo(48266)) then
--~             if MikSBT then
--~                 MikSBT.DisplayMessage(L["Wrong Presence!?"] , "Info", true, 255, 25, 25)
--~             else
--~                 DocsUI:p(L["Wrong Presence!?"])
--~             end
--~         elseif select(5, GetTalentInfo(1,3))==5 and select(5, GetTalentInfo(3,3))==5 and not UnitBuff("player", GetSpellInfo(48263)) then
--~             if MikSBT then
--~                 MikSBT.DisplayMessage(L["Wrong Presence!?"] , "Info", true, 255, 25, 25)
--~             else
--~                 DocsUI:p(L["Wrong Presence!?"])
--~             end
--~         end
--~     end
end

