--[[
	Copyright (c) 2009, dr_AllCOM3
    All rights reserved.

    You're allowed to use this addon, free of monetary charge,
    but you are not allowed to modify, alter, or redistribute
    this addon without express, written permission of the author.
]]

local L = LibStub( "AceLocale-3.0" ):GetLocale( "DocsUI" )
local LSM = LibStub( "LibSharedMedia-3.0" )

local temp, temp2
local db = {}

local anchor = CreateFrame( "Frame", nil, UIParent )
anchor:SetWidth( 80 )
anchor:SetHeight( 5*6+5+5*2  )
anchor:SetPoint( "TOP", UIParent, "BOTTOM", 0, 500 )
DocsUI.oUFanchor = anchor

--[[ Print ]]
local function p( text )
    ChatFrame3:AddMessage( tostring( text ) )
end

local UnitAura = UnitAura
local strlower = strlower
local format = string.format

local name, _ = UnitName("player")
local _, class = UnitClass("player")
local maxLevel = 80
local aggro = {}
local aggroWarning = true
local colors

local function lightenColor( t )
    local r, g, b = unpack( t )
    local modifier = 1.5
    
    return { r+( 1-r )/modifier, g+( 1-g )/modifier, b+( 1-b )/modifier }
end

local function round(num, idp) -- Round a number. number, decimal places
    local mult = 10^(idp or 0)
    return math.floor(num * mult + 0.5) / mult
end

local function siValue(val) --Short numbers
    if val >= 10000000 then 
        return format("%.1fm", val / 1000000) 
    elseif val >= 1000000 then
        return format("%.2fm", val / 1000000) 
    elseif val >= 100000 then
        return format("%.0fk", val / 1000) 
    elseif val >= 10000 then
        return format("%.1fk", val / 1000) 
    else
        return val
    end
end

local function Hex(r, g, b)
    if type(r) == "table" then
        if r.r then r, g, b = r.r, r.g, r.b else r, g, b = unpack(r) end
    end
    return string.format("|cff%02x%02x%02x", r*255, g*255, b*255)
end

local function createImprovedStatusbar( self ) -- Creates a texture that acts like a statusbar. This gets rid of the ugly tiling.
    local statusbar = CreateFrame( "Frame", nil, self )
    
    local function UpdateBar( self )
        local fraction = .01
        local range = self.MaxVal-self.MinVal 
        local value = self.Value-self.MinVal
        local barsize = self.Dim or 100
        
        if range>0 and value>0 and range>=value then
            fraction = value/range
        else
            fraction = .0001
        end
        
        if self.Orientation=="VERTICAL" then 
            self.Bar:SetHeight( barsize*fraction )
            self.Bar:SetTexCoord( 0, 1, 1-fraction, 1 )
        else 
            self.Bar:SetWidth( barsize*fraction ) 
            self.Bar:SetTexCoord( 0, fraction, 0, 1 )
        end
    end
    
    local function UpdateSize( self ) 
        if self.Orientation=="VERTICAL" then
            self.Dim = self:GetHeight()
        else
            self.Dim = self:GetWidth()
        end
        
        UpdateBar( self ) 
    end
    
    local function SetValue( self, value )
        if value>=self.MinVal and value<=self.MaxVal then self.Value = value end
        
        UpdateBar( self ) 
    end
    
    local function GetValue( self, value )
        return self.Value
    end
    
    local function SetMinMaxValues( self, minval, maxval )
        if maxval>minval then
            self.MinVal = minval
            self.MaxVal = maxval
        else 
            self.MinVal = 0
            self.MaxVal = 100
        end
        
        if self.Value>self.MaxVal then self.Value = self.MaxVal
        elseif self.Value<self.MinVal then self.Value = self.MinVal end
        
        UpdateBar( self ) 
    end
    
    local function SetOrientation( self, orientation ) 
        if orientation=="VERTICAL" then
            self.Orientation = orientation
            self.Bar:ClearAllPoints()
            self.Bar:SetPoint( "BOTTOMLEFT" )
            self.Bar:SetPoint( "BOTTOMRIGHT" )
        else
            self.Orientation = "HORIZONTAL"
            self.Bar:ClearAllPoints()
            self.Bar:SetPoint( "TOPLEFT" )
            self.Bar:SetPoint( "BOTTOMLEFT" )
        end
        
        UpdateSize( self )
    end
    
    local function SetStatusBarColor( self, r, g, b, a )
        self.Bar:SetVertexColor( r, g, b, a )
    end
    
    local function GetStatusBarColor( self )
        return self.Bar:GetVertexColor()
    end
    
    local function SetStatusBarTexture( self, texture )
        self.Bar:SetTexture( texture )
    end
    
    local function GetStatusBarTexture( self )
        return self.Bar:GetTexture()
    end
    
    statusbar.SetFrameLevelOld = statusbar.SetFrameLevel
    local function SetFrameLevel( self, level )
        self:SetFrameLevelOld( level )
    end
    
    statusbar.SetWidthOld = statusbar.SetWidth
    local function SetWidth( self, value )
        self:SetWidthOld( value )
        
        SetOrientation( self, self.Orientation )
        UpdateSize( self ) 
    end
    
    statusbar.SetHeightOld = statusbar.SetHeight
    local function SetHeight( self, value )
        self:SetHeightOld( value )
        
        SetOrientation( self, self.Orientation )
        UpdateSize( self ) 
    end
    
    if not statusbar.Bar then
        temp = statusbar:CreateTexture( nil, "ARTWORK" )
    else
        temp = statusbar.Bar
    end
    statusbar.Bar = temp
    
    statusbar.Value = 100
    statusbar.MinVal = 0
    statusbar.MaxVal = 100
    statusbar.Orientation = "HORIZONTAL"
    statusbar.UpdateSize = UpdateSize
    statusbar.SetValue = SetValue
    statusbar.GetValue = GetValue
    statusbar.SetMinMaxValues = SetMinMaxValues
    statusbar.SetOrientation = SetOrientation
    statusbar.SetStatusBarColor = SetStatusBarColor
    statusbar.GetStatusBarColor = GetStatusBarColor
    statusbar.SetStatusBarTexture = SetStatusBarTexture
    statusbar.GetStatusBarTexture = GetStatusBarTexture
    statusbar.SetFrameLevel = SetFrameLevel
    statusbar.SetWidth = SetWidth
    statusbar.SetHeight = SetHeight
    
    --statusbar:SetPoint( "TOP", self, "TOP", 0, 0 )
    --statusbar:SetWidth( db.width )
    --statusbar:SetHeight( db.height )
    statusbar:SetOrientation( "HORIZONTAL" )
    --statusbar:SetStatusBarTexture( LSM:Fetch( "statusbar", db.texture ) )
    
    statusbar:SetScript( "OnSizeChanged", UpdateSize )
    UpdateSize( statusbar )
    
    return statusbar
end

local function PostUpdateHealth( health, unit, min, max )
    
end

local function PostUpdatePower( power, unit, min, max )
    if UnitIsDead( unit ) or UnitIsGhost( unit ) then
        power:SetValue(0)
    end
end

local function PostUpdateThreat(self)
    local isTanking, status, threatpct, rawthreatpct, threatvalue = UnitDetailedThreatSituation("player", "target")
    
    if tonumber(threatpct) and not isTanking then
        if threatpct>=90 then
            if aggroWarning then
                if MikSBT then MikSBT.DisplayMessage("Aggro!" , "Notification", true, 255, 125, 0) end
            end
            aggroWarning = false
        elseif threatpct>=80 then
            aggroWarning = true
        else
            aggroWarning = true
        end
        
        self:Show()
        self:SetValue(threatpct)
    else
        self:Hide()
    end
end

local menu = function(self)
	local unit = self.unit:sub(1, -2)
	local cunit = self.unit:gsub("^%l", string.upper)

	if(cunit == 'Vehicle') then
		cunit = 'Pet'
	end

	if(unit == "party" or unit == "partypet") then
		ToggleDropDownMenu(1, nil, _G["PartyMemberFrame"..self.id.."DropDown"], "cursor", 0, 0)
	elseif(_G[cunit.."FrameDropDown"]) then
		ToggleDropDownMenu(1, nil, _G[cunit.."FrameDropDown"], "cursor", 0, 0)
	end
end

--[[ Settings are applied here ]]
local UnitSpecific = {}

local function Shared( self, unit )
    self.menu = menu
    self.colors = colors
    
    self:SetScript( "OnEnter", UnitFrame_OnEnter )
    self:SetScript( "OnLeave", UnitFrame_OnLeave )
    self:RegisterForClicks( "AnyDown" )
    
    DocsUI:setBorder( self )
end

function UnitSpecific.player( self, ... )
    Shared( self, ... )
    
    local bg = self:CreateTexture( nil, "BORDER" )
    bg:SetAllPoints( self )
    bg:SetTexture( db.bg )
    bg:SetVertexColor( db.bgColor.r, db.bgColor.g, db.bgColor.b, db.bgColor.a )
    self.bg = bg
    
    local castbar
    do
        castbar = createImprovedStatusbar( self )
        castbar:SetWidth( db.width*1 )
        castbar:SetHeight( db.heightSmall )
        castbar:SetStatusBarTexture( LSM:Fetch( "statusbar", db.texture ) )
        castbar:SetStatusBarColor( db.castColor.r, db.castColor.g, db.castColor.b, db.castColor.a )
        castbar:SetPoint( "TOP", self, "TOP", 0, 0 )
        castbar:UpdateSize( castbar )
        castbar:SetFrameLevel( self:GetFrameLevel()+1 )
        self.Castbar = castbar
        
        castbar.bg = castbar:CreateTexture( nil, "BORDER" )
        castbar.bg:SetAllPoints( castbar )
        castbar.bg:SetTexture( LSM:Fetch( "statusbar", db.texture ) )
        castbar.bg:SetVertexColor( db.castColor.r, db.castColor.g, db.castColor.b, db.castColor.a )
        castbar.bg:SetAlpha( db.barAlpha )
        
        castbar.Time = castbar:CreateFontString( nil, "OVERLAY", "GameFontNormal" )
        --castbar.Time:SetWidth( db.height*0.5 )
        --castbar.Time:SetHeight( db.height*0.5 )
        castbar.Time:SetPoint( "BOTTOMRIGHT", castbar, "TOPRIGHT", -db.spacingFixed*2, 0 )
        castbar.Time:SetFont( LSM:Fetch( "font", db.fontb ), db.fontsize )
        castbar.Time:SetShadowColor( 0,0,0,1 )
        castbar.Time:SetShadowOffset( 1,-1 )
        castbar.Time:SetJustifyH( "LEFT" )
        castbar.Time:SetJustifyV( "CENTER" )
        castbar.Time:SetTextColor( unpack( lightenColor( { db.castColor.r, db.castColor.g, db.castColor.b, db.castColor.a } ) ) )
        
        castbar.Icon = castbar:CreateTexture(nil)
        castbar.Icon:SetWidth( db.height*0.5 )
        castbar.Icon:SetHeight( db.height*0.5 )
        castbar.Icon:SetPoint( "RIGHT", castbar.Time, "LEFT", -db.spacingFixed*2, 0 )
        
        castbar.SafeZone = castbar:CreateTexture( nil, "ARTWORK" )
        castbar.SafeZone:SetTexture( db.bg )
        castbar.SafeZone:SetVertexColor( 1, 0.2, 0, 0.5 )
    end
    
    local health = createImprovedStatusbar( self )
    health:SetWidth( db.width )
    health:SetHeight( db.height/2 )
    health:SetPoint( "TOP", castbar, "BOTTOM", 0, 0 )
    health:SetStatusBarTexture( LSM:Fetch( "statusbar", db.texture ) )
    health:SetStatusBarColor( db.fgColor.r, db.fgColor.g, db.fgColor.b, db.fgColor.a )
    health:UpdateSize( health )
    health:SetFrameLevel( self:GetFrameLevel()+1 )
    
    health.frequentUpdates = true
    --health.colorClass = true
    --health.colorSmooth = false
    health.Smooth = true
    self.Health = health
    health.PostUpdate = PostUpdateHealth
    
    local power = createImprovedStatusbar( self )
    power:SetWidth( db.width*1 )
    power:SetHeight( db.heightSmall )
    power:SetStatusBarTexture( LSM:Fetch( "statusbar", db.texture ) )
    power:SetPoint( "TOP", health, "BOTTOM", 0, 0 )
    power:UpdateSize( power )
    power:SetFrameLevel( self:GetFrameLevel()+1 )
    
    local powerbg = power:CreateTexture( nil, "BORDER" )
    powerbg:SetAllPoints( power )
    powerbg:SetTexture( LSM:Fetch( "statusbar", db.texture ) )
    powerbg:SetAlpha( db.barAlpha )
    power.bg = powerbg
    
    local powertext2 = health:CreateFontString( nil, "OVERLAY", "GameFontNormal" )
    powertext2:SetPoint( "RIGHT", health, "RIGHT", -db.spacingFixed*2, 0 )
    powertext2:SetFont( LSM:Fetch("font", db.fontb), db.fontsize )
    powertext2:SetShadowColor( 0,0,0,1 )
    powertext2:SetShadowOffset( 1,-1 )
    powertext2:SetJustifyH( "RIGHT" )
    powertext2:SetJustifyV( "CENTER" )
    powertext2.frequentUpdates = 0.1
    self:Tag( powertext2, "[DocsUI:powerpercent]" )
    
    local powertext = power:CreateFontString( nil, "OVERLAY", "GameFontNormal" )
    powertext:SetPoint( "RIGHT", powertext2, "LEFT", -db.spacingFixed*2, 0 )
    powertext:SetFont( LSM:Fetch("font", db.font), db.fontsize )
    powertext:SetShadowColor( 0,0,0,1 )
    powertext:SetShadowOffset( 1,-1 )
    powertext:SetJustifyH( "RIGHT" )
    powertext:SetJustifyV( "CENTER" )
    powertext.frequentUpdates = 0.1
    self:Tag( powertext, "[DocsUI:power]" )
    
    power.colorPower = true
    power.frequentUpdates = true
    power.frequentUpdates = true
    power.Smooth = true
    power.value = powertext
    self.Power = power
    self.PostUpdate = PostUpdatePower
    
    local healthtext = health:CreateFontString( nil, "OVERLAY", "GameFontNormal" )
    healthtext:SetPoint( "LEFT", health, "LEFT", db.spacingFixed*2, 0 )
    healthtext:SetFont( LSM:Fetch( "font", db.fontb ), db.fontsize )
    healthtext:SetShadowColor( 0,0,0,1 )
    healthtext:SetShadowOffset( 1,-1 )
    healthtext:SetJustifyH( "LEFT" )
    healthtext:SetJustifyV( "CENTER" )
    healthtext:SetTextColor( db.fontColor.r,db.fontColor.g,db.fontColor.b,db.fontColor.a )
    health.value = healthtext
    healthtext.frequentUpdates = 0.1
    self:Tag( healthtext, "[DocsUI:healthpercent]" )
    
    local healthtext2 = health:CreateFontString( nil, "OVERLAY", "GameFontNormal" )
    healthtext2:SetPoint( "LEFT", healthtext, "RIGHT", db.spacingFixed*2, 0 )
    healthtext2:SetFont( LSM:Fetch("font", db.font), db.fontsize-1 )
    healthtext2:SetShadowColor( 0,0,0,1 )
    healthtext2:SetShadowOffset( 1,-1 )
    healthtext2:SetJustifyH("CENTER")
    healthtext2:SetJustifyV("CENTER")
    healthtext2:SetTextColor( db.fontColor.r,db.fontColor.g,db.fontColor.b,db.fontColor.a )
    health.value2 = healthtext2
    healthtext2.frequentUpdates = 0.1
    self:Tag( healthtext2, "[DocsUI:health]" )
    
    local combat = health:CreateTexture( nil, "OVERLAY" )
    combat:SetHeight( db.iconSize*0.8 )
    combat:SetWidth( db.iconSize*0.8 )
    combat:SetPoint( "CENTER", self, "BOTTOMLEFT", 0, 0 )
    self.Combat = combat
    
    local resting = health:CreateTexture( nil, "OVERLAY" )
    resting:SetHeight( db.iconSize*0.8 )
    resting:SetWidth( db.iconSize*0.8 )
    resting:SetPoint( "CENTER", self, "BOTTOM", 0, 0 )
    self.Resting = resting
    
    local LFDRole = health:CreateTexture( nil, "OVERLAY" )
    LFDRole:SetHeight( db.iconSize*0.6 )
    LFDRole:SetWidth( db.iconSize*0.6 )
    LFDRole:SetPoint( "CENTER", self, "LEFT", 0, 0 )
    self.LFDRole = LFDRole
    
    local leader = health:CreateTexture( nil, "OVERLAY" )
    leader:SetHeight( db.iconSize*0.6 )
    leader:SetWidth( db.iconSize*0.6 )
    leader:SetPoint( "CENTER", self, "TOPLEFT", 0, 0 )
    leader:SetTexture( "Interface\GroupFrame\UI-Group-LeaderIcon" )
    self.Leader = leader
    
    local ricon = health:CreateTexture( nil, "OVERLAY" )
    ricon:SetHeight( db.iconSize*1 )
    ricon:SetWidth( db.iconSize*1 )
    ricon:SetPoint("CENTER", self, "TOP", 0, 0)
    self.RaidIcon = ricon
    
    self:SetSize( db.width, castbar:GetHeight()+health:GetHeight()+power:GetHeight() )
end

function UnitSpecific.target( self, ... )
    Shared( self, ... )
    
    local bg = self:CreateTexture( nil, "BORDER" )
    bg:SetAllPoints( self )
    bg:SetTexture( db.bg )
    bg:SetVertexColor( db.bgColor.r, db.bgColor.g, db.bgColor.b, db.bgColor.a )
    self.bg = bg
    
    -- Threat
    local threat = createImprovedStatusbar( self, self.threat )
    threat:SetWidth( db.width*1 )
    threat:SetHeight( db.heightSmall )
    threat:SetPoint( "TOP", self, "TOP", 0, 0 )
    threat:SetStatusBarColor( db.threatColor.r, db.threatColor.g, db.threatColor.b, db.threatColor.a )
    threat:UpdateSize( threat )
    threat:SetFrameLevel( self:GetFrameLevel()+1 )
    threat:SetStatusBarTexture(LSM:Fetch("statusbar", db.texture))
    tinsert( aggro, threat )
    
    local threatbg = threat:CreateTexture(nil, "BORDER")
    threatbg:SetAllPoints( threat )
    threatbg:SetTexture(LSM:Fetch("statusbar", db.texture))
    threatbg:SetVertexColor(db.bgColor.r, db.bgColor.g, db.bgColor.b, db.bgColor.a)
    threat.bg = tbg
    
    local health = createImprovedStatusbar( self, self.Health )
    health:SetWidth( db.width )
    health:SetHeight( db.height/2 )
    health:SetPoint( "TOP", threat, "BOTTOM", 0, 0 )
    health:SetStatusBarTexture( LSM:Fetch("statusbar", db.texture) )
    health:SetStatusBarColor( db.fgColor.r, db.fgColor.g, db.fgColor.b, db.fgColor.a )
    health:UpdateSize( health )
    health:SetFrameLevel( self:GetFrameLevel()+1 )
    health.frequentUpdates = true
    --health.colorTapping = true
    --health.colorHappiness = false
    --health.colorClass = true
    --health.colorReaction = true
    --health.colorSmooth = false
    health.Smooth = true
    health.PostUpdate = PostUpdateHealth
    self.Health = health
    
    local power = createImprovedStatusbar( self )
    power:SetWidth( db.width*1 )
    power:SetHeight( db.heightSmall )
    power:SetStatusBarTexture( LSM:Fetch("statusbar", db.texture) )
    power:SetPoint( "TOP", health, "BOTTOM", 0, 0 )
    power:UpdateSize( power )
    power:SetFrameLevel( self:GetFrameLevel()+1 )
    
    local powerbg = power:CreateTexture( nil, "BORDER" )
    powerbg:SetAllPoints( power )
    powerbg:SetTexture( LSM:Fetch("statusbar", db.texture) )
    powerbg:SetAlpha( db.barAlpha )
    power.bg = powerbg
    --[[
    local powertext = power:CreateFontString( nil, "OVERLAY", "GameFontNormal" )
    powertext:SetPoint( "LEFT", power, "LEFT", db.spacingFixed*2, 0 )
    powertext:SetFont( LSM:Fetch( "font", db.font ), db.fontsize )
    powertext:SetShadowColor( 0,0,0,1 )
    powertext:SetShadowOffset( 1,-1 )
    powertext:SetJustifyH( "LEFT" )
    powertext:SetJustifyV( "CENTER" )
    powertext.frequentUpdates = 0.1
    power.value = powertext
    self:Tag( powertext, "[DocsUI:power]" )
    
    local powertext2 = health:CreateFontString( nil, "OVERLAY", "GameFontNormal" )
    powertext2:SetPoint( "BOTTOM", power, "TOP", 0, 0 )
    --powertext2:SetPoint( "CENTER", health, "RIGHT", -db.width*0.5, 0 )
    powertext2:SetFont( LSM:Fetch("font", db.fontb), db.fontsize )
    powertext2:SetShadowColor( 0,0,0,1 )
    powertext2:SetShadowOffset( 1,-1 )
    powertext2:SetJustifyH( "LEFT" )
    powertext2:SetJustifyV( "CENTER" )
    powertext2.frequentUpdates = 0.1
    self:Tag( powertext2, "[DocsUI:powerpercent]" )
    ]]
    power.colorPower = true
    power.frequentUpdates = true
    power.Smooth = true
    self.Power = power
    self.PostUpdate = PostUpdatePower
    
    local healthtext = health:CreateFontString( nil, "OVERLAY", "GameFontNormal" )
    healthtext:SetPoint( "RIGHT", health, "RIGHT", -db.spacingFixed*2, 0 )
    healthtext:SetFont( LSM:Fetch( "font", db.fontb ), db.fontsize )
    healthtext:SetShadowColor( 0,0,0,1 )
    healthtext:SetShadowOffset( 1,-1 )
    healthtext:SetJustifyH( "RIGHT" )
    healthtext:SetJustifyV( "CENTER" )
    healthtext:SetTextColor( db.fontColor.r,db.fontColor.g,db.fontColor.b,db.fontColor.a )
    health.value = healthtext
    healthtext.frequentUpdates = 0.1
    self:Tag( healthtext, "[DocsUI:healthpercent]" )
    
    local healthtext2 = health:CreateFontString( nil, "OVERLAY", "GameFontNormal" )
    healthtext2:SetPoint( "RIGHT", healthtext, "LEFT", -db.spacingFixed*2, 0 )
    healthtext2:SetFont( LSM:Fetch("font", db.font), db.fontsize-1 )
    healthtext2:SetShadowColor( 0,0,0,1 )
    healthtext2:SetShadowOffset( 1,-1 )
    healthtext2:SetJustifyH("CENTER")
    healthtext2:SetJustifyV("CENTER")
    healthtext2:SetTextColor( db.fontColor.r,db.fontColor.g,db.fontColor.b,db.fontColor.a )
    health.value2 = healthtext2
    healthtext2.frequentUpdates = 0.1
    self:Tag( healthtext2, "[DocsUI:health]" )
    
    local name = health:CreateFontString( nil, "OVERLAY", "GameFontNormal" )
    name:SetPoint( "LEFT", health, "LEFT", db.spacingFixed*2, 0 )
    name:SetPoint( "RIGHT", healthtext2, "LEFT", 0, 0 )
    name:SetFont( LSM:Fetch( "font", db.fonti ), db.fontsize-1 )
    name:SetJustifyH( "LEFT" )
    name:SetJustifyV( "CENTER" )
    name:SetShadowColor( 0,0,0,1 )
    name:SetShadowOffset( 1,-1 )
    name:SetTextColor( db.fontColor.r,db.fontColor.g,db.fontColor.b,db.fontColor.a )
    self:Tag( name, "[DocsUI:name]" )
    self.Name = name
    
    local castbar = {}
    do
        local height = db.height/2
        
        castbar = createImprovedStatusbar( frame )
        castbar:SetWidth( db.width-height )
        castbar:SetHeight( height )
        castbar:SetStatusBarTexture( LSM:Fetch("statusbar", db.texture) )
        castbar:SetStatusBarColor( db.castColor.r, db.castColor.g, db.castColor.b, db.castColor.a )
        castbar:SetPoint( "BOTTOM", DocsUI.oUFanchor, "TOP", 0, 75 )
        castbar:UpdateSize( castbar )
        --castbar:SetFrameLevel( health:GetFrameLevel()+1 )
        DocsUI:setBorder( castbar )
        self.Castbar = castbar
        
        castbar.Icon = castbar:CreateTexture( nil )
        castbar.Icon:SetWidth( height )
        castbar.Icon:SetHeight( height )
        castbar.Icon:SetPoint( "RIGHT", castbar, "LEFT", -4, 0 )
        
        castbar.bg = castbar:CreateTexture( nil, "BORDER" )
        castbar.bg:SetAllPoints( castbar )
        castbar.bg:SetTexture( LSM:Fetch("statusbar", db.texture) )
        castbar.bg:SetVertexColor( db.castColor.r, db.castColor.g, db.castColor.b )
        castbar.bg:SetAlpha( db.barAlpha )
        
        castbar.Time = castbar:CreateFontString( nil, "OVERLAY", "GameFontNormal" )
        castbar.Time:SetHeight( height )
        castbar.Time:SetPoint( "RIGHT", castbar, "RIGHT", -db.spacingFixed, 0 )
        castbar.Time:SetFont( LSM:Fetch( "font", db.fontb ), db.fontsize )
        castbar.Time:SetShadowColor( 0,0,0,1 )
        castbar.Time:SetShadowOffset( 1,-1 )
        castbar.Time:SetJustifyH( "RIGHT" )
        castbar.Time:SetJustifyV( "CENTER" )
        castbar.Time:SetTextColor( unpack( lightenColor( { db.castColor.r, db.castColor.g, db.castColor.b, db.castColor.a } ) ) )
        
        castbar.Text = castbar:CreateFontString( nil, "OVERLAY", "GameFontNormal" )
        castbar.Text:SetHeight( height )
        castbar.Text:SetPoint( "LEFT", castbar, "LEFT", db.spacingFixed, 0 )
        castbar.Text:SetPoint( "RIGHT", castbar.Time, "LEFT", 0, 0 )
        castbar.Text:SetFont( LSM:Fetch("font", db.font), db.fontsize-2 )
        castbar.Text:SetJustifyH( "LEFT" )
        castbar.Text:SetJustifyV( "CENTER" )
        castbar.Text:SetShadowColor( 0,0,0,1 )
        castbar.Text:SetShadowOffset( 1,-1 )
        castbar.Text:SetTextColor( unpack( lightenColor( { db.castColor.r, db.castColor.g, db.castColor.b, db.castColor.a } ) ) )
        
        castbar:SetScript( "OnShow", function( self )
            self:SetPoint( "BOTTOM", DocsUI.oUFanchor, "TOP", 0, 75 )
            
            local name, subText, text, texture, startTime, endTime, isTradeSkill, castID, notInterruptible = UnitCastingInfo( "target" )
            
            if notInterruptible==nil then
                name, subText, text, texture, startTime, endTime, isTradeSkill, notInterruptible = UnitChannelInfo( "target" )
            end
            
            if name then
                for i=1,#db.castbarHider do
                    if db.castbarHider[i]==strlower( name ) then
                        self:Hide()
                        return
                    end
                end
                
                self:Show()
            end
            
            if notInterruptible then
                self:SetStatusBarColor( 0.5,0.5,0.5 )
                self.bg:SetVertexColor( 0.5,0.5,0.5 )
                self.Text:SetTextColor( 0.6,0.6,0.6)
            else
                self:SetStatusBarColor( db.castColor.r, db.castColor.g, db.castColor.b, db.castColor.a )
                self.bg:SetVertexColor( db.castColor.r, db.castColor.g, db.castColor.b )
                self.Text:SetTextColor( unpack( lightenColor( { db.castColor.r, db.castColor.g, db.castColor.b, db.castColor.a } ) ) )
            end
        end )
        castbar:SetScript( "OnHide", function( self ) name:Show() end )
    end
    
    local ricon = health:CreateTexture( nil, "OVERLAY" )
    ricon:SetHeight( db.iconSize*1 )
    ricon:SetWidth( db.iconSize*1 )
    ricon:SetPoint("CENTER", self, "TOP", 0, 0)
    self.RaidIcon = ricon
    
    local LFDRole = health:CreateTexture( nil, "OVERLAY" )
    LFDRole:SetHeight( db.iconSize*0.6 )
    LFDRole:SetWidth( db.iconSize*0.6 )
    LFDRole:SetPoint( "CENTER", self, "LEFT", 0, 0 )
    self.LFDRole = LFDRole
    
    local leader = health:CreateTexture( nil, "OVERLAY" )
    leader:SetHeight( db.iconSize*0.6 )
    leader:SetWidth( db.iconSize*0.6 )
    leader:SetPoint( "CENTER", self, "TOPLEFT", 0, 0 )
    leader:SetTexture( "Interface\GroupFrame\UI-Group-LeaderIcon" )
    self.Leader = leader
    
    self:SetSize( db.width, threat:GetHeight()+health:GetHeight()+power:GetHeight() )
    --[[
    
    -- Combo Points
    self.CPoints = {}
    for i = 1, 5 do
        self.CPoints[i] = self.Health:CreateTexture(nil, "OVERLAY", self)
        self.CPoints[i]:SetWidth(db.height)
        self.CPoints[i]:SetHeight(db.height)
        if i==1 then self.CPoints[i]:SetTexture(db.textureComboPoint1) end
        if i==2 then self.CPoints[i]:SetTexture(db.textureComboPoint2) end
        if i==3 then self.CPoints[i]:SetTexture(db.textureComboPoint3) end
        if i==4 then self.CPoints[i]:SetTexture(db.textureComboPoint4) end
        if i==5 then self.CPoints[i]:SetTexture(db.textureComboPoint5) end
    end
    self.CPoints[1]:SetPoint("BOTTOMLEFT", self, "TOPLEFT", 0, db.spacingFixed*2)
    self.CPoints[2]:SetPoint("BOTTOMLEFT", self.CPoints[1], "BOTTOMRIGHT", 0, 0)
    self.CPoints[3]:SetPoint("BOTTOMLEFT", self.CPoints[2], "BOTTOMRIGHT", 0, 0)
    self.CPoints[4]:SetPoint("BOTTOMLEFT", self.CPoints[3], "BOTTOMRIGHT", 0, 0)
    self.CPoints[5]:SetPoint("BOTTOMLEFT", self.CPoints[4], "BOTTOMRIGHT", 0, 0)
    
    end
    ]]
end

function UnitSpecific.pet( self, ... )
    Shared( self, ... )
    
    local width = db.width2
    
    local health = createImprovedStatusbar( self )
    health:SetWidth( width )
    health:SetHeight( db.height/2 )
    health:SetPoint( "TOP" )
    health:SetStatusBarTexture( LSM:Fetch("statusbar", db.texture) )
    health:SetStatusBarColor( db.fgColor.r, db.fgColor.g, db.fgColor.b, db.fgColor.a )
    health:UpdateSize( health )
    
    local healthbg = health:CreateTexture( nil, "BORDER" )
    healthbg:SetAllPoints( health )
    healthbg:SetTexture( LSM:Fetch("statusbar", db.texture) )
    healthbg:SetVertexColor( db.bgColor.r, db.bgColor.g, db.bgColor.b, db.bgColor.a )
    health.bg = healthbg
    
    health.frequentUpdates = true
    --health.colorClass = true
    --health.colorSmooth = false
    health.Smooth = true
    self.Health = health
    health.PostUpdate = PostUpdateHealth
    
    local name = health:CreateFontString( nil, "OVERLAY", "GameFontNormal" )
    name:SetWidth( width )
    name:SetHeight( db.height/2 )
    name:SetPoint( "LEFT", health, "LEFT", db.spacingFixed*2, 0 )
    name:SetFont( LSM:Fetch( "font", db.fonti ), db.fontsize-1 )
    name:SetJustifyH( "LEFT" )
    name:SetJustifyV( "CENTER" )
    name:SetShadowColor( 0,0,0,1 )
    name:SetShadowOffset( 1,-1 )
    name:SetTextColor( db.fontColor.r,db.fontColor.g,db.fontColor.b,db.fontColor.a )
    self:Tag( name, "[DocsUI:name]" )
    self.Name = name
    
    self:SetSize( width, health:GetHeight() )
    
    --[[
    -- Health bar
    local hp = CreateFrame"StatusBar"
    hp:SetWidth(db.width/3)
    hp:SetHeight(db.height)
    hp:SetStatusBarTexture(LSM:Fetch("statusbar", db.texture))
    hp:SetParent(self)
    hp:SetPoint("BOTTOMLEFT")
    DocsUI:setBorder(hp)
    
    hp.frequentUpdates = true
    hp.colorDisconnected = false
    hp.colorTapping = false
    hp.colorHappiness = false
    hp.colorClass = true
    hp.colorReaction = true
    hp.colorSmooth = false
    hp.Smooth = true

    self.Health = hp
    self.PostUpdateHealth = PostUpdateHealth
    
    hp:SetScript("OnShow", function() if type(DocsUI.UpdateAuras)=="function" then DocsUI:UpdateAuras("pet") end end) -- To prevent wierd pet buff stuff.
    hp:SetScript("OnHide", function() if type(DocsUI.UpdateAuras)=="function" then DocsUI:UpdateAuras("pet") end end)
    
    -- Health bar background
    local hpbg = hp:CreateTexture(nil, "BORDER")
    hpbg:SetAllPoints(hp)
    hpbg:SetAlpha(1)
    hpbg:SetTexture(LSM:Fetch("statusbar", db.texture))
    hpbg:SetVertexColor(db.bgColor.r, db.bgColor.g, db.bgColor.b, db.bgColor.a)
    hp.bg = hpbg
    
    -- Unit name
    local name = hp:CreateFontString(nil, "OVERLAY", "GameFontNormal")
    name:SetWidth(db.width/2)
    name:SetHeight(db.height)
    name:SetPoint("LEFT", hp, "LEFT", db.spacingFixed, 0)
    name:SetFont(LSM:Fetch("font", db.font), db.fontsize)
    name:SetJustifyH("LEFT")
    name:SetJustifyV("CENTER")
    name:SetShadowColor(0,0,0,1)
    name:SetShadowOffset(1,-1)
    name:SetTextColor(db.fontColor.r,db.fontColor.g,db.fontColor.b,db.fontColor.a)
    
    self:Tag(name, "[name]")
    self.Name = name
    ]]
end

function UnitSpecific.targettarget( self, ... )
    Shared( self, ... )
    
    local width = db.width2
    
    local health = createImprovedStatusbar( self, self.Health )
    health:SetWidth( width )
    health:SetHeight( db.height/2 )
    health:SetPoint( "TOP" )
    health:SetStatusBarTexture( LSM:Fetch("statusbar", db.texture) )
    health:SetStatusBarColor( db.fgColor.r, db.fgColor.g, db.fgColor.b, db.fgColor.a )
    health:UpdateSize( health )
    health.frequentUpdates = true
    --health.colorTapping = true
    --health.colorHappiness = false
    --health.colorClass = true
    --health.colorReaction = true
    --health.colorSmooth = false
    health.Smooth = true
    health.PostUpdate = PostUpdateHealth
    self.Health = health
    
    local healthbg = health:CreateTexture( nil, "BORDER" )
    healthbg:SetAllPoints( health )
    healthbg:SetTexture( LSM:Fetch("statusbar", db.texture) )
    healthbg:SetVertexColor( db.bgColor.r, db.bgColor.g, db.bgColor.b, db.bgColor.a )
    health.bg = healthbg
    
    local name = health:CreateFontString( nil, "OVERLAY", "GameFontNormal" )
    name:SetWidth( width )
    name:SetHeight( db.height/2 )
    name:SetPoint( "LEFT", health, "LEFT", db.spacingFixed*2, 0 )
    name:SetFont( LSM:Fetch( "font", db.fonti ), db.fontsize-1 )
    name:SetJustifyH( "LEFT" )
    name:SetJustifyV( "CENTER" )
    name:SetShadowColor( 0,0,0,1 )
    name:SetShadowOffset( 1,-1 )
    name:SetTextColor( db.fontColor.r,db.fontColor.g,db.fontColor.b,db.fontColor.a )
    self:Tag( name, "[DocsUI:name]" )
    self.Name = name
    
    local ricon = health:CreateTexture( nil, "OVERLAY" )
    ricon:SetHeight( db.iconSize*1 )
    ricon:SetWidth( db.iconSize*1 )
    ricon:SetPoint( "RIGHT", self, "RIGHT", 0, 0 )
    self.RaidIcon = ricon
    
    self:SetSize( width, health:GetHeight() )
end

function UnitSpecific.focus( self, ... )
    Shared( self, ... )
    
    local width = db.width
    local height = db.height/2
    
    local bg = self:CreateTexture( nil, "BORDER" )
    bg:SetAllPoints( self )
    bg:SetTexture( db.bg )
    bg:SetVertexColor( db.bgColor.r, db.bgColor.g, db.bgColor.b, db.bgColor.a )
    self.bg = bg
    
    local name
    local castbar
    do
        castbar = createImprovedStatusbar( self )
        castbar:SetWidth( width )
        castbar:SetHeight( db.heightSmall )
        castbar:SetStatusBarTexture( LSM:Fetch( "statusbar", db.texture ) )
        castbar:SetStatusBarColor( db.castColor.r, db.castColor.g, db.castColor.b, db.castColor.a )
        castbar:SetPoint( "TOP", self, "TOP", 0, 0 )
        castbar:UpdateSize( castbar )
        castbar:SetFrameLevel( self:GetFrameLevel()+1 )
        self.Castbar = castbar
        
        castbar.bg = castbar:CreateTexture( nil, "BORDER" )
        castbar.bg:SetAllPoints( castbar )
        castbar.bg:SetTexture( LSM:Fetch( "statusbar", db.texture ) )
        castbar.bg:SetVertexColor( db.castColor.r, db.castColor.g, db.castColor.b, db.castColor.a )
        castbar.bg:SetAlpha( db.barAlpha )
        
        castbar.Icon = castbar:CreateTexture( nil, "OVERLAY" )
        castbar.Icon:SetWidth( db.height*0.5 )
        castbar.Icon:SetHeight( db.height*0.5 )
        castbar.Icon:SetPoint( "TOPLEFT", castbar, "BOTTOMLEFT", db.spacingFixed*2, 0 )
        
        castbar.Time = castbar:CreateFontString( nil, "OVERLAY", "GameFontNormal" )
        castbar.Time:SetWidth( width )
        castbar.Time:SetHeight( db.height*0.5 )
        castbar.Time:SetPoint( "TOPRIGHT", castbar, "BOTTOMRIGHT", -db.spacingFixed*2, 0 )
        castbar.Time:SetFont( LSM:Fetch( "font", db.fontb ), db.fontsize )
        castbar.Time:SetShadowColor( 0,0,0,1 )
        castbar.Time:SetShadowOffset( 1,-1 )
        castbar.Time:SetJustifyH( "RIGHT" )
        castbar.Time:SetJustifyV( "CENTER" )
        castbar.Time:SetTextColor( unpack( lightenColor( { db.castColor.r, db.castColor.g, db.castColor.b, db.castColor.a } ) ) )
        
        castbar.Text = castbar:CreateFontString( nil, "OVERLAY", "GameFontNormal" )
        castbar.Text:SetWidth( width )
        castbar.Text:SetHeight( db.height*0.5 )
        castbar.Text:SetPoint( "TOPLEFT", castbar.Icon, "TOPRIGHT", 0, 0 )
        --castbar.Text:SetPoint( "TOPRIGHT", castbar.Time, "TOPLEFT", 0, 0 )
        castbar.Text:SetFont( LSM:Fetch("font", db.font), db.fontsize-2 )
        castbar.Text:SetJustifyH( "LEFT" )
        castbar.Text:SetJustifyV( "CENTER" )
        castbar.Text:SetShadowColor( 0,0,0,1 )
        castbar.Text:SetShadowOffset( 1,-1 )
        castbar.Text:SetTextColor( unpack( lightenColor( { db.castColor.r, db.castColor.g, db.castColor.b, db.castColor.a } ) ) )
        
        castbar.SafeZone = castbar:CreateTexture( nil, "ARTWORK" )
        castbar.SafeZone:SetTexture( db.bg )
        castbar.SafeZone:SetVertexColor( 1, 0.2, 0, 0.5 )
        
        castbar:SetScript( "OnShow", function( self )
            name:Hide()
            
            local name, subText, text, texture, startTime, endTime, isTradeSkill, castID, notInterruptible = UnitCastingInfo( "target" )
            
            if notInterruptible==nil then
                name, subText, text, texture, startTime, endTime, isTradeSkill, notInterruptible = UnitChannelInfo( "target" )
            end
            
            if name then
                for i=1,#db.castbarHider do
                    if db.castbarHider[i]==strlower( name ) then
                        self:Hide()
                        return
                    end
                end
                
                self:Show()
            end
            
            if notInterruptible then
                self:SetStatusBarColor( 0.5,0.5,0.5 )
                self.bg:SetVertexColor( 0.5,0.5,0.5 )
                self.Text:SetTextColor( 0.6,0.6,0.6)
            else
                self:SetStatusBarColor( db.castColor.r, db.castColor.g, db.castColor.b, db.castColor.a )
                self.bg:SetVertexColor( db.castColor.r, db.castColor.g, db.castColor.b )
                self.Text:SetTextColor( unpack( lightenColor( { db.castColor.r, db.castColor.g, db.castColor.b, db.castColor.a } ) ) )
            end
        end )
        castbar:SetScript( "OnHide", function( self ) name:Show() end )
    end
    
    local health = createImprovedStatusbar( self, self.Health )
    health:SetWidth( width )
    health:SetHeight( height )
    health:SetPoint( "TOP", castbar, "BOTTOM", 0, 0 )
    health:SetStatusBarTexture( LSM:Fetch("statusbar", db.texture) )
    health:SetStatusBarColor( db.fgColor.r, db.fgColor.g, db.fgColor.b, db.fgColor.a )
    health:UpdateSize( health )
    health:SetFrameLevel( self:GetFrameLevel()+1 )
    health.frequentUpdates = true
    --health.colorTapping = true
    --health.colorHappiness = false
    --health.colorClass = true
    --health.colorReaction = true
    --health.colorSmooth = false
    health.Smooth = true
    health.PostUpdate = PostUpdateHealth
    self.Health = health
    
    local power = createImprovedStatusbar( self )
    power:SetWidth( width )
    power:SetHeight( db.heightSmall )
    power:SetStatusBarTexture( LSM:Fetch("statusbar", db.texture) )
    power:SetPoint( "TOP", health, "BOTTOM", 0, 0 )
    power:UpdateSize( power )
    power:SetFrameLevel( self:GetFrameLevel()+1 )
    
    local powerbg = power:CreateTexture( nil, "BORDER" )
    powerbg:SetAllPoints( power )
    powerbg:SetTexture( LSM:Fetch("statusbar", db.texture) )
    powerbg:SetAlpha( db.barAlpha )
    power.bg = powerbg
    
    power.colorPower = true
    power.frequentUpdates = true
    power.Smooth = true
    self.Power = power
    self.PostUpdate = PostUpdatePower
    
    name = health:CreateFontString( nil, "OVERLAY", "GameFontNormal" )
    name:SetPoint( "LEFT", health, "LEFT", db.spacingFixed*2, 0 )
    name:SetPoint( "RIGHT", health, "RIGHT", 0, 0 )
    name:SetFont( LSM:Fetch( "font", db.fonti ), db.fontsize-1 )
    name:SetJustifyH( "LEFT" )
    name:SetJustifyV( "CENTER" )
    name:SetShadowColor( 0,0,0,1 )
    name:SetShadowOffset( 1,-1 )
    name:SetTextColor( db.fontColor.r,db.fontColor.g,db.fontColor.b,db.fontColor.a )
    self:Tag( name, "[DocsUI:name]" )
    self.Name = name
    
    local ricon = health:CreateTexture( nil, "OVERLAY" )
    ricon:SetHeight( db.iconSize*1 )
    ricon:SetWidth( db.iconSize*1 )
    ricon:SetPoint("CENTER", self, "TOP", 0, 0)
    self.RaidIcon = ricon
    
    self:SetAttribute( "initial-height", castbar:GetHeight()+health:GetHeight()+power:GetHeight() )
    
    self:SetSize( width, castbar:GetHeight()+health:GetHeight()+power:GetHeight() )
end

function UnitSpecific.focustarget( self, ... )
    Shared( self, ... )
    
    local width = db.width2
    
    local health = createImprovedStatusbar( self, self.Health )
    health:SetWidth( width )
    health:SetHeight( db.height/2 )
    health:SetPoint( "TOP" )
    health:SetStatusBarTexture( LSM:Fetch("statusbar", db.texture) )
    health:SetStatusBarColor( db.fgColor.r, db.fgColor.g, db.fgColor.b, db.fgColor.a )
    health:UpdateSize( health )
    health.frequentUpdates = true
    --health.colorTapping = true
    --health.colorHappiness = false
    --health.colorClass = true
    --health.colorReaction = true
    --health.colorSmooth = false
    health.Smooth = true
    health.PostUpdate = PostUpdateHealth
    self.Health = health
    
    local healthbg = health:CreateTexture( nil, "BORDER" )
    healthbg:SetAllPoints( health )
    healthbg:SetTexture( LSM:Fetch("statusbar", db.texture) )
    healthbg:SetVertexColor( db.bgColor.r, db.bgColor.g, db.bgColor.b, db.bgColor.a )
    health.bg = healthbg
    
    local name = health:CreateFontString( nil, "OVERLAY", "GameFontNormal" )
    name:SetWidth( width )
    name:SetHeight( db.height/2 )
    name:SetPoint( "LEFT", health, "LEFT", db.spacingFixed*2, 0 )
    name:SetFont( LSM:Fetch( "font", db.fonti ), db.fontsize-1 )
    name:SetJustifyH( "LEFT" )
    name:SetJustifyV( "CENTER" )
    name:SetShadowColor( 0,0,0,1 )
    name:SetShadowOffset( 1,-1 )
    name:SetTextColor( db.fontColor.r,db.fontColor.g,db.fontColor.b,db.fontColor.a )
    self:Tag( name, "[DocsUI:name]" )
    self.Name = name
    
    local ricon = health:CreateTexture( nil, "OVERLAY" )
    ricon:SetHeight( db.iconSize*1 )
    ricon:SetWidth( db.iconSize*1 )
    ricon:SetPoint( "RIGHT", self, "RIGHT", 0, 0 )
    self.RaidIcon = ricon
    
    self:SetSize( width, health:GetHeight() )
end

function UnitSpecific.boss( self, ... )
    Shared( self, ... )
    
    local width = db.width2/1
    local height = db.height/2
    
    local health = createImprovedStatusbar( self, self.Health )
    health:SetWidth( width )
    health:SetHeight( height )
    health:SetPoint( "TOP" )
    health:SetStatusBarTexture( LSM:Fetch("statusbar", db.texture) )
    health:SetStatusBarColor( db.fgColor.r, db.fgColor.g, db.fgColor.b, db.fgColor.a )
    health:UpdateSize( health )
    health.frequentUpdates = true
    --health.colorTapping = true
    --health.colorHappiness = false
    --health.colorClass = true
    --health.colorReaction = true
    --health.colorSmooth = false
    health.Smooth = true
    health.PostUpdate = PostUpdateHealth
    self.Health = health
    
    local healthbg = health:CreateTexture( nil, "BORDER" )
    healthbg:SetAllPoints( health )
    healthbg:SetTexture( LSM:Fetch("statusbar", db.texture) )
    healthbg:SetVertexColor( db.bgColor.r, db.bgColor.g, db.bgColor.b, db.bgColor.a )
    health.bg = healthbg
    
    local name = health:CreateFontString( nil, "OVERLAY", "GameFontNormal" )
    name:SetWidth( width )
    name:SetHeight( db.height/2 )
    name:SetPoint( "LEFT", health, "LEFT", db.spacingFixed*2, 0 )
    name:SetFont( LSM:Fetch( "font", db.fonti ), db.fontsize-1 )
    name:SetJustifyH( "LEFT" )
    name:SetJustifyV( "CENTER" )
    name:SetShadowColor( 0,0,0,1 )
    name:SetShadowOffset( 1,-1 )
    name:SetTextColor( db.fontColor.r,db.fontColor.g,db.fontColor.b,db.fontColor.a )
    self:Tag( name, "[DocsUI:name]" )
    self.Name = name
    
    local ricon = health:CreateTexture( nil, "OVERLAY" )
    ricon:SetHeight( db.iconSize*1 )
    ricon:SetWidth( db.iconSize*1 )
    ricon:SetPoint( "RIGHT", self, "RIGHT", 0, 0 )
    self.RaidIcon = ricon
    
    self:SetSize( health:GetWidth(), health:GetHeight() )
end

function UnitSpecific.maintank( self, ... ) -- raid is used
    Shared( self, ... )
    
    local width = db.width2/1
    local height = db.height/2
    
    local health = createImprovedStatusbar( self, self.Health )
    health:SetWidth( width )
    health:SetHeight( height )
    health:SetPoint( "TOP" )
    health:SetStatusBarTexture( LSM:Fetch("statusbar", db.texture) )
    health:SetStatusBarColor( db.fgColor.r, db.fgColor.g, db.fgColor.b, db.fgColor.a )
    health:UpdateSize( health )
    health.frequentUpdates = true
    --health.colorTapping = true
    --health.colorHappiness = false
    --health.colorClass = true
    --health.colorReaction = true
    --health.colorSmooth = false
    health.Smooth = true
    health.PostUpdate = PostUpdateHealth
    self.Health = health
    
    local healthbg = health:CreateTexture( nil, "BORDER" )
    healthbg:SetAllPoints( health )
    healthbg:SetTexture( LSM:Fetch("statusbar", db.texture) )
    healthbg:SetVertexColor( db.bgColor.r, db.bgColor.g, db.bgColor.b, db.bgColor.a )
    health.bg = healthbg
    
    local name = health:CreateFontString( nil, "OVERLAY", "GameFontNormal" )
    name:SetWidth( width )
    name:SetHeight( height )
    name:SetPoint( "LEFT", health, "LEFT", db.spacingFixed*2, 0 )
    name:SetFont( LSM:Fetch( "font", db.fonti ), db.fontsize-1 )
    name:SetJustifyH( "LEFT" )
    name:SetJustifyV( "CENTER" )
    name:SetShadowColor( 0,0,0,1 )
    name:SetShadowOffset( 1,-1 )
    name:SetTextColor( db.fontColor.r,db.fontColor.g,db.fontColor.b,db.fontColor.a )
    self:Tag( name, "[DocsUI:name]" )
    self.Name = name
    
    local ricon = health:CreateTexture( nil, "OVERLAY" )
    ricon:SetHeight( db.iconSize*1 )
    ricon:SetWidth( db.iconSize*1 )
    ricon:SetPoint( "RIGHT", self, "RIGHT", 0, 0 )
    self.RaidIcon = ricon
    
    self:SetSize( health:GetWidth(), health:GetHeight() )
end

function UnitSpecific.party( self, ... ) -- raid is used
    if not db.partyShow then return end
    
    Shared( self, ... )
    
    local width = db.width
    local height = db.height/2
    
    local bg = self:CreateTexture( nil, "BORDER" )
    bg:SetAllPoints( self )
    bg:SetTexture( db.bg )
    bg:SetVertexColor( db.bgColor.r, db.bgColor.g, db.bgColor.b, db.bgColor.a )
    self.bg = bg
    
    local health = createImprovedStatusbar( self, self.Health )
    health:SetWidth( width )
    health:SetHeight( height )
    health:SetPoint( "TOP" )
    health:SetStatusBarTexture( LSM:Fetch( "statusbar", db.texture ) )
    health:SetStatusBarColor( db.fgColor.r, db.fgColor.g, db.fgColor.b, db.fgColor.a )
    health:UpdateSize( health )
    health.frequentUpdates = true
    --health.colorTapping = true
    --health.colorHappiness = false
    --health.colorClass = true
    --health.colorReaction = true
    --health.colorSmooth = false
    health.Smooth = true
    health.PostUpdate = PostUpdateHealth
    self.Health = health
    
    local healthbg = health:CreateTexture( nil, "BORDER" )
    healthbg:SetAllPoints( health )
    healthbg:SetTexture( LSM:Fetch( "statusbar", db.texture ) )
    healthbg:SetVertexColor( db.bgColor.r, db.bgColor.g, db.bgColor.b, db.bgColor.a )
    health.bg = healthbg
    
    local power = createImprovedStatusbar( self )
    power:SetWidth( width )
    power:SetHeight( db.heightSmall )
    power:SetStatusBarTexture( LSM:Fetch("statusbar", db.texture) )
    power:SetPoint( "TOP", health, "BOTTOM", 0, 0 )
    power:UpdateSize( power )
    power:SetFrameLevel( self:GetFrameLevel()+1 )
    
    local powerbg = power:CreateTexture( nil, "BORDER" )
    powerbg:SetAllPoints( power )
    powerbg:SetTexture( LSM:Fetch("statusbar", db.texture) )
    powerbg:SetAlpha( db.barAlpha )
    power.bg = powerbg
    
    power.colorPower = true
    power.frequentUpdates = true
    power.Smooth = true
    self.Power = power
    self.PostUpdate = PostUpdatePower
    
    local name = health:CreateFontString( nil, "OVERLAY", "GameFontNormal" )
    name:SetPoint( "LEFT", health, "LEFT", db.spacingFixed*2, 0 )
    name:SetFont( LSM:Fetch( "font", db.fonti ), db.fontsize )
    name:SetJustifyH( "LEFT" )
    name:SetJustifyV( "CENTER" )
    name:SetShadowColor( 0,0,0,1 )
    name:SetShadowOffset( 1,-1 )
    name:SetTextColor( db.fontColor.r,db.fontColor.g,db.fontColor.b,db.fontColor.a )
    self:Tag( name, "[DocsUI:name]" )
    self.Name = name
    
    local leader = health:CreateTexture( nil, "OVERLAY" )
    leader:SetHeight( db.iconSize*0.6 )
    leader:SetWidth( db.iconSize*0.6 )
    leader:SetPoint( "CENTER", health, "TOPLEFT", 0, 0 )
    leader:SetTexture( "Interface\GroupFrame\UI-Group-LeaderIcon" )
    self.Leader = leader
    
    local LFDRole = health:CreateTexture( nil, "OVERLAY" )
    LFDRole:SetHeight( db.iconSize*0.6 )
    LFDRole:SetWidth( db.iconSize*0.6 )
    LFDRole:SetPoint( "RIGHT", health, "RIGHT", 0, 0 )
    self.LFDRole = LFDRole
    
    local ricon = health:CreateTexture( nil, "OVERLAY" )
    ricon:SetHeight( db.iconSize*1 )
    ricon:SetWidth( db.iconSize*1 )
    ricon:SetPoint( "RIGHT", health, "RIGHT", 0, 0 )
    self.RaidIcon = ricon
    
    self:SetSize( width, health:GetHeight()+power:GetHeight() )
end

function UnitSpecific.raid( self, ... )
    if not db.raidShow then return end
    
    Shared( self, ... )
    
    local width = db.width2/1
    local height = db.height/2
    
    local health = createImprovedStatusbar( self, self.Health )
    health:SetWidth( width )
    health:SetHeight( height )
    health:SetPoint( "TOP" )
    health:SetStatusBarTexture( LSM:Fetch( "statusbar", db.texture ) )
    health:SetStatusBarColor( db.fgColor.r, db.fgColor.g, db.fgColor.b, db.fgColor.a )
    health:UpdateSize( health )
    health.frequentUpdates = true
    --health.colorTapping = true
    --health.colorHappiness = false
    --health.colorClass = true
    --health.colorReaction = true
    --health.colorSmooth = false
    health.Smooth = true
    health.PostUpdate = PostUpdateHealth
    self.Health = health
    
    local healthbg = health:CreateTexture( nil, "BORDER" )
    healthbg:SetAllPoints( health )
    healthbg:SetTexture( LSM:Fetch( "statusbar", db.texture ) )
    healthbg:SetVertexColor( db.bgColor.r, db.bgColor.g, db.bgColor.b, db.bgColor.a )
    health.bg = healthbg
    
    local name = health:CreateFontString( nil, "OVERLAY", "GameFontNormal" )
    name:SetWidth( width )
    name:SetHeight( height )
    name:SetPoint( "LEFT", health, "LEFT", db.spacingFixed*2, 0 )
    name:SetFont( LSM:Fetch( "font", db.font ), db.fontsize-4 )
    name:SetJustifyH( "LEFT" )
    name:SetJustifyV( "CENTER" )
    name:SetShadowColor( 0,0,0,1 )
    name:SetShadowOffset( 1,-1 )
    name:SetTextColor( db.fontColor.r,db.fontColor.g,db.fontColor.b,db.fontColor.a )
    self:Tag( name, "[DocsUI:name]" )
    self.Name = name
    
    local LFDRole = health:CreateTexture( nil, "OVERLAY" )
    LFDRole:SetHeight( db.iconSize*0.6 )
    LFDRole:SetWidth( db.iconSize*0.6 )
    LFDRole:SetPoint( "RIGHT", self, "RIGHT", 0, 0 )
    self.LFDRole = LFDRole
    
    local ricon = health:CreateTexture( nil, "OVERLAY" )
    ricon:SetHeight( db.iconSize*1 )
    ricon:SetWidth( db.iconSize*1 )
    ricon:SetPoint( "RIGHT", self, "RIGHT", 0, 0 )
    self.RaidIcon = ricon
    
    self:SetSize( health:GetWidth(), health:GetHeight() )
end

-- UNIT_THREAT_LIST_UPDATE
function DocsUI:UNIT_THREAT_LIST_UPDATE()
    for i=1,#aggro do
        PostUpdateThreat(aggro[i])
    end
end

-- Spawn oUF
function DocsUI:SpawnOuf()
    db = DocsUI.db.profile
    
    DocsUI:RegisterEvent("UNIT_THREAT_LIST_UPDATE")
    
    colors = setmetatable({
        health = {.45, .73, .27},
        power = setmetatable({
            ["MANA"] = {.27, .53, .73},
            ["RAGE"] = {.73, .27, .27},
        }, {__index = oUF.colors.power}),
        runes = {{0.77, 0.12, 0.23}, {0.3, 0.8, 0.1}, {0, 0.4, 0.7}, {0.51, 0.23, 0.65}},
    }, {__index = oUF.colors})
    
    oUF.Tags["DocsUI:name"] = function( unit )
        local reaction = UnitReaction( unit, "player" )
        local _, class = UnitClass( unit )
        local player = UnitIsPlayer( unit )
        local r, g, b = 1, 1, 1
        
        if ( UnitIsTapped( unit ) and not UnitIsTappedByPlayer( unit ) ) or not UnitIsConnected( unit ) or UnitIsDeadOrGhost( unit ) then
            r, g, b = 0.5, 0.5, 0.5
        elseif player and class then
            r, g, b = RAID_CLASS_COLORS[class]
        elseif not player and reaction then
            r, g, b = unpack( _COLORS.reaction[reaction] )
        elseif UnitFactionGroup( unit ) and UnitIsEnemy( unit, "player" ) and UnitIsPVP( unit ) then
            r, g, b = 1, 0, 0
        end
        
        return ( "%s%s|r" ):format( Hex( r, g, b ), UnitName( unit ) )
    end

    oUF.Tags["DocsUI:health"] = function( unit )
        local reaction = UnitReaction( unit, "player" )
        local _, class = UnitClass( unit )
        local player = UnitIsPlayer( unit )
        local r, g, b = 1, 1, 1
        
        if ( UnitIsTapped( unit ) and not UnitIsTappedByPlayer( unit ) ) or not UnitIsConnected( unit ) then
            r, g, b = 0.5, 0.5, 0.5
        elseif player and class then
            r, g, b = RAID_CLASS_COLORS[class]
        elseif not player and reaction then
            r, g, b = unpack( _COLORS.reaction[reaction] )
        elseif UnitFactionGroup( unit ) and UnitIsEnemy( unit, "player" ) and UnitIsPVP( unit ) then
            r, g, b = 1, 0, 0
        end
        
        if type(r)=="table" then r, g, b = r.r, r.g, r.b end
        
        return ( "%s%s|r" ):format( Hex( lightenColor( { r, g, b } ) ), siValue( UnitHealth( unit ) ) )--lightenColor( { r, g, b } )
    end

    oUF.Tags["DocsUI:healthpercent"] = function( unit )
        local reaction = UnitReaction( unit, "player" )
        local _, class = UnitClass( unit )
        local player = UnitIsPlayer( unit )
        local r, g, b = 1, 1, 1
        
        if ( UnitIsTapped( unit ) and not UnitIsTappedByPlayer( unit ) ) or not UnitIsConnected( unit ) then
            r, g, b = 0.5, 0.5, 0.5 --db.fontColor.r, db.fontColor.g, db.fontColor.b
        elseif player and class then
            r, g, b = RAID_CLASS_COLORS[class]
        elseif not player and reaction then
            r, g, b = unpack( _COLORS.reaction[reaction] )
        elseif UnitFactionGroup( unit ) and UnitIsEnemy( unit, "player" ) and UnitIsPVP( unit ) then
            r, g, b = 1, 0, 0
        end
        
        local percent = ( UnitHealth( unit )/UnitHealthMax( unit ) )*100
        if percent>1 then
            percent = floor( percent )
        else
            percent = round( percent, 2 )
        end
        
        return ( "%s%d%s|r" ):format( Hex( r, g, b ), percent, "%" )
    end

    oUF.Tags["DocsUI:power"] = function( unit )
        local powerType, powerTypeString = UnitPowerType( unit )
        local power = UnitPower(unit)
        
        if powerTypeString=="MANA" and power>0 and not UnitIsDeadOrGhost(unit) then
            local colors = _COLORS.power
            
            return ( "%s%s|r" ):format( Hex( lightenColor( colors[powerTypeString] ) or lightenColor( colors["RUNES"] ) ), siValue( power ) )
        end
    end

    oUF.Tags["DocsUI:powerpercent"] = function( unit )
        local powerType, powerTypeString = UnitPowerType( unit )
        local power = UnitPower( unit )
        
        if power>0 and not UnitIsDeadOrGhost(unit) then
            local colors = _COLORS.power
            
            if powerTypeString=="MANA" then
                local percent = ( UnitPower( unit )/UnitPowerMax( unit ) )*100
                if percent>1 then
                    percent = floor( percent )
                else
                    percent = round( percent, 2 )
                end
                
                return ( "%s%d|r" ):format( Hex( colors[powerTypeString] or colors["RUNES"] ), percent )
            else
                return ( "%s%d|r" ):format( Hex( colors[powerTypeString] or colors["RUNES"] ), power )
            end
        end
    end
    
    oUF:RegisterStyle( "DocsUI", Shared )
    for unit,layout in next,UnitSpecific do
        -- Capitalize the unit name, so it looks better.
        oUF:RegisterStyle( "DocsUI - "..unit:gsub( "^%l", string.upper ), layout )
    end
    
    -- A small helper to change the style into a unit specific, if it exists.
    local spawnHelper = function( self, unit, ... )
        if strfind( unit, "boss" ) then unit = "boss" end
        
        if UnitSpecific[unit] then
            self:SetActiveStyle( "DocsUI - "..unit:gsub( "^%l", string.upper ) )
            local object = self:Spawn( unit )
            object:SetPoint( ... )
            return object
        else
            self:SetActiveStyle( "DocsUI" )
            local object = self:Spawn( unit )
            object:SetPoint( ... )
            return object
        end
    end
    
    oUF:Factory( function( self )
        anchor:SetHeight( db.heightSmall  )
        anchor:ClearAllPoints()
        anchor:SetPoint( "TOP", UIParent, "BOTTOM", 0, db.vertical )
        
        local player = spawnHelper( self, "player", "TOPRIGHT", DocsUI.oUFanchor, "TOPLEFT", -db.spacing*8, 0 )
        DocsUI.oUFplayer = player
        
        local target = spawnHelper( self, "target", "TOPLEFT", DocsUI.oUFanchor, "TOPRIGHT", db.spacing*8, 0 )
        DocsUI.oUFtarget = target
        
        local pet = spawnHelper( self, "pet", "RIGHT", player, "LEFT", -db.spacing*8, 0 )
        DocsUI.oUFpet = pet
        
        local targettarget = spawnHelper( self, "targettarget", "LEFT", target, "RIGHT", db.spacing*8, 0 )
        DocsUI.oUFtargettarget = targettarget
        
        local focus = spawnHelper( self, "focus", "TOPLEFT", target, "BOTTOMLEFT", 0, -db.spacing*8 )
        local focustarget = spawnHelper( self, "focustarget", "LEFT", focus, "RIGHT", db.spacing*8, 0 )
        
        self:SetActiveStyle( "DocsUI - Maintank" )
        local maintank = self:SpawnHeader( nil, nil, "raid,party,solo",
            "groupFilter", "MAINTANK",
            "showRaid", true,
            "yOffSet", -db.spacing*2,
            "point", "TOP",
            "maxColumns", 1,
            "unitsPerColumn", 5,
            "template", "DocsUI_maintanktarget"
        )
        maintank:SetPoint( "TOPLEFT", player, "BOTTOMLEFT", 0, -db.spacing*8 )
        
        local mainassist = self:SpawnHeader( nil, nil, "raid,party,solo",
            "groupFilter", "MAINASSIST",
            "showRaid", true,
            "yOffSet", -db.spacing*2,
            "point", "TOP",
            "maxColumns", 1,
            "unitsPerColumn", 5,
            "template", "DocsUI_maintanktarget"
        )
        mainassist:SetPoint( "TOPLEFT", maintank, "BOTTOMLEFT", 0, -db.spacing*2 )
        
        local boss = {}
        for i=1,MAX_BOSS_FRAMES do
            if i==1 then
                boss[i] = spawnHelper( self, "boss"..i, "TOPLEFT", mainassist, "BOTTOMLEFT", 0, -db.spacing*2 )
            else
                boss[i] = spawnHelper( self, "boss"..i, "TOPLEFT", boss[i-1], "BOTTOMLEFT", 0, -db.spacing*2 )
            end
        end
        
        CompactPartyFrame:Hide()
        self:SetActiveStyle( "DocsUI - Party" )
        local party = self:SpawnHeader( nil, nil, "party",
            "showParty", true,
            "yOffSet", -db.spacingFixed*2,
            "point", "TOP",
            "maxColumns", 1,
            "unitsPerColumn", 5
        )
        party:SetPoint( "TOPLEFT", UIParent, "TOPLEFT", db.edgeDistance, -db.edgeDistance )
        
        CompactRaidFrameContainer:Hide()
        self:SetActiveStyle( "DocsUI - Raid" )
        local raid = self:SpawnHeader( nil, nil, "raid",
            "groupBy", "CLASS",
            "groupingOrder", "DEATHKNIGHT,DRUID,HUNTER,MAGE,PALADIN,PRIEST,ROGUE,SHAMAN,WARLOCK,WARRIOR",
            "showRaid", true,
            "yOffSet", -db.spacingFixed*2,
            "point", "TOP",
            "maxColumns", 1,
            "unitsPerColumn", 40
        )
        raid:SetPoint( "TOPLEFT", UIParent, "TOPLEFT", db.edgeDistance, -db.edgeDistance )
        
        if type(DocsUI.SpawnBuffFrames)=="function" then DocsUI:SpawnBuffFrames() end
    end )
end