﻿--[[
	Copyright (c) 2009, dr_AllCOM3
    All rights reserved.

    You're allowed to use this addon, free of monetary charge,
    but you are not allowed to modify, alter, or redistribute
    this addon without express, written permission of the author.
]]

local L = LibStub("AceLocale-3.0"):GetLocale("DocsUI")
local LSM = LibStub("LibSharedMedia-3.0")
local db
local temp, temp2 = nil

local optionsGeneral = {
    name = L["Layout"],
    desc = L["Layout Options."],
    type = "group",
    order = 1,
    args = {
        header4 = {
            name = L["Position"],
            type = "header",
            order = 90,
        },
        horizontal = {
            name = L["Horizontal"],
            desc = L["Move the unit frames left and right."],
            type = "range",
            min = -500,
            max = 500,
            step = 1,
            order = 91,
            get=function() return db.horizontal end,
            set=function(self, val)
                    db.horizontal = val
                end,
        },
        vertical = {
            name = L["Vertical"],
            desc = L["Move the unit frames up and down."],
            type = "range",
            min = 0,
            max = 1000,
            step = 1,
            order = 92,
            get=function() return db.vertical end,
            set=function(self, val)
                    db.vertical = val
                end,
        },
        
        header1 = {
            name = L["Layout"],
            type = "header",
            order = 80,
        },
        width = {
            name = L["Width"],
            desc = L["General width."],
            type = "range",
            min = 50,
            max = 500,
            step = 1,
            order = 81,
            get=function() return db.width end,
            set=function(self, val)
                    db.width = val
                end,
        },
        height = {
            name = L["Height"],
            desc = L["General height."],
            type = "range",
            min = 5,
            max = 200,
            step = 1,
            order = 82,
            get=function() return db.height end,
            set=function(self, val)
                    db.height = val
                end,
        },
        heightSmall = {
            name = L["Height Small"],
            desc = L["General height of small objects."],
            type = "range",
            min = 1,
            max = 100,
            step = 1,
            order = 83,
            get=function() return db.heightSmall end,
            set=function(self, val)
                    db.heightSmall = val
                end,
        },
        
        header2 = {
            name = L["Font"],
            type = "header",
            order = 20,
        },
        fontsize = {
            name = L["Font Size"],
            desc = L["Size of fonts."],
            type = "range",
            min = 5,
            max = 50,
            step = 1,
            order = 21,
            get=function() return db.fontsize end,
            set=function(self, val)
                    db.fontsize = val
                end,
        },
        font = {
            type = "select",
            name = L["Font"],
            desc = L["Select a font for normal text."],
            values = LSM:HashTable("font"),
            dialogControl = "LSM30_Font",
            order = 22,
            get=function() return db.font end,
            set=function(self, val)
                    db.font = val
                end,
        },
        fontb = {
            type = "select",
            name = L["Font Bold"],
            desc = L["Select a font for bold text."],
            values = LSM:HashTable("font"),
            dialogControl = "LSM30_Font",
            order = 23,
            get=function() return db.fontb end,
            set=function(self, val)
                    db.fontb = val
                end,
        },
        fonti = {
            type = "select",
            name = L["Font Italic"],
            desc = L["Select a font for italic text."],
            values = LSM:HashTable("font"),
            dialogControl = "LSM30_Font",
            order = 25,
            get=function() return db.fonti end,
            set=function(self, val)
                    db.fonti = val
                end,
        },
        fontColor = {
            name = L["Font Color"],
            desc = L["Selects the font color."],
            type = "color",
            hasAlpha = true,
            order = 24,
            get=function(i)
                    local c = db.fontColor
                    return c.r, c.g, c.b, c.a
                end,
            set=function(i, r,g,b,a) 
                    db.fontColor = {["r"] = r, ["g"] = g, ["b"] = b, ["a"] = a}
                end,
        },
        
        header3 = {
            name = L["Textures"],
            type = "header",
            order = 30,
        },
        texture = {
            type = "select",
            name = L["Statusbar"],
            desc = L["Set the statusbar texture."],
            values = LSM:HashTable("statusbar"),
            dialogControl = "LSM30_Statusbar",
            order = 31,
            get=function() return db.texture end,
            set=function(self, val)
                    db.texture = val
                end,
        },
        fgColor = {
            name = L["Foreground Color"],
            desc = L["Selects the foreground color."],
            type = "color",
            hasAlpha = true,
            order = 32,
            get=function(i)
                    local c = db.fgColor
                    return c.r, c.g, c.b, c.a
                end,
            set=function(i, r,g,b,a) 
                    db.fgColor = {["r"] = r, ["g"] = g, ["b"] = b, ["a"] = a}
                end,
        },
        bgColor = {
            name = L["Background Color"],
            desc = L["Selects the background color."],
            type = "color",
            hasAlpha = true,
            order = 33,
            get=function(i)
                    local c = db.bgColor
                    return c.r, c.g, c.b, c.a
                end,
            set=function(i, r,g,b,a) 
                    db.bgColor = {["r"] = r, ["g"] = g, ["b"] = b, ["a"] = a}
                end,
        },
    }
}

local optionsUnitFrames = {
    name = L["Unit Frames"],
    desc = L["Unit Frame options."],
    type = "group",
    order = 2,
    args = {
        raidShow = {
            type = "toggle",
            name = L["Show Raid Frame"],
            desc = L["Toggle the raid frame."],
            order = 2,
            get=function() return db.raidShow end,
            set=function()
                    db.raidShow = not db.raidShow
                end,
        },
        partyShow = {
            type = "toggle",
            name = L["Show Party Frame"],
            desc = L["Toggle the party frame."],
            order = 1,
            get=function() return db.partyShow end,
            set=function()
                    db.partyShow = not db.partyShow
                end,
        },
        runeBarsShow = {
            type = "toggle",
            name = L["Show Rune Bars"],
            desc = L["Toggle the rune bars."],
            order = 5,
            get=function() return db.runeBarsShow end,
            set=function()
                    db.runeBarsShow = not db.runeBarsShow
                end,
        },
        
        buffSize = {
            name = L["Buff Size"],
            desc = L["Size of the buffs."],
            type = "range",
            min = 5,
            max = 100,
            step = 1,
            order = 11,
            get=function() return db.buffSize end,
            set=function(self, val)
                    db.buffSize = val
                end,
        },
        
        header2 = {
            name = L["Castbar Hider"],
            type = "header",
            order = 20,
        },
        description1 = {
            name = L["castbarHiderDescription"],
            type = "description",
            order = 21,
        },
        castbarHider = {
            type = "select",
            name = L["Delete Spell Name"],
            desc = L["Click a spell name to delete it from the list."],
            style = "dropdown",
            width = "double",
            order = 22,
            values = function()
                    return db.castbarHider
                end,
            get=function()  end,
            set=function(self, i)
                    if db.castbarHider[i]==L["Close"] then
                        return
                    else
                        tremove(db.castbarHider, i)
                    end
                end,
        },
        castbarHiderName = {
            name = L["Add Spell Name"],
            desc = L["Input spell name and press enter to add it."],
            type = "input",
            width = "double",
            order = 23,
            get=function()  end,
            set=function(self, val)
                    val = strlower(val)
                    tinsert(db.castbarHider, val)
                end,
        },
        
    }
}

local optionsActionbars = {
    name = L["Actionbars"],
    desc = L["Actionbar options."],
    type = "group",
    order = 3,
    args = {
        header1 = {
            name = L["Main Bar"],
            type = "header",
            order = 10,
        },
        mainBar = {
            type = "toggle",
            name = L["Show"],
            desc = L["Toggle the bar."],
            order = 11,
            disabled = true,
            get=function() return db.mainBar end,
            set=function()
                    db.mainBar = not db.mainBar
                end,
        },
        mainBarMouseover = {
            type = "toggle",
            name = L["Show On Mouseover"],
            desc = L["Show the bar only when the mouse is over it."],
            order = 12,
            get=function() return db.mainBarMouseover end,
            set=function()
                    db.mainBarMouseover = not db.mainBarMouseover
                end,
        },
        mainBarScale = {
            name = L["Bar Scale"],
            desc = L["Size of the bar."],
            type = "range",
            min = 0.1,
            max = 3,
            step = 0.05,
            order = 13,
            get=function() return db.mainBarScale end,
            set=function(self, val)
                    db.mainBarScale = val
                end,
        },
        
        header2 = {
            name = L["Second Bar"],
            type = "header",
            order = 20,
        },
        secondBar = {
            type = "toggle",
            name = L["Show"],
            desc = L["Toggle the bar."],
            order = 21,
            get=function() return db.secondBar end,
            set=function()
                    db.secondBar = not db.secondBar
                    
                    if db.secondBar then
                        SHOW_MULTI_ACTIONBAR_1 = 1
                    else
                        SHOW_MULTI_ACTIONBAR_1 = nil
                    end
                    
                    InterfaceOptions_UpdateMultiActionBars()
                    MultiActionBar_Update()
                end,
        },
        secondBarMouseover = {
            type = "toggle",
            name = L["Show On Mouseover"],
            desc = L["Show the bar only when the mouse is over it."],
            order = 22,
            get=function() return db.secondBarMouseover end,
            set=function()
                    db.secondBarMouseover = not db.secondBarMouseover
                end,
        },
        secondBarScale = {
            name = L["Bar Scale"],
            desc = L["Size of the bar."],
            type = "range",
            min = 0.1,
            max = 3,
            step = 0.05,
            order = 23,
            get=function() return db.secondBarScale end,
            set=function(self, val)
                    db.secondBarScale = val
                end,
        },
        
        header3 = {
            name = L["Third Bar"],
            type = "header",
            order = 30,
        },
        thirdBar = {
            type = "toggle",
            name = L["Show"],
            desc = L["Toggle the bar."],
            order = 31,
            get=function() return db.thirdBar end,
            set=function()
                    db.thirdBar = not db.thirdBar
                    
                    if db.thirdBar then
                        SHOW_MULTI_ACTIONBAR_2 = 1
                    else
                        SHOW_MULTI_ACTIONBAR_2 = nil
                    end
                    
                    InterfaceOptions_UpdateMultiActionBars()
                    MultiActionBar_Update()
                end,
        },
        thirdBarMouseover = {
            type = "toggle",
            name = L["Show On Mouseover"],
            desc = L["Show the bar only when the mouse is over it."],
            order = 32,
            get=function() return db.thirdBarMouseover end,
            set=function()
                    db.thirdBarMouseover = not db.thirdBarMouseover
                end,
        },
        thirdBarScale = {
            name = L["Bar Scale"],
            desc = L["Size of the bar."],
            type = "range",
            min = 0.1,
            max = 3,
            step = 0.05,
            order = 33,
            get=function() return db.thirdBarScale end,
            set=function(self, val)
                    db.thirdBarScale = val
                end,
        },
        
        header4 = {
            name = L["Right Bar"],
            type = "header",
            order = 40,
        },
        right1Bar = {
            type = "toggle",
            name = L["Show"],
            desc = L["Toggle the bar."],
            order = 41,
            get=function() return db.right1Bar end,
            set=function()
                    db.right1Bar = not db.right1Bar
                    
                    if db.right1Bar then
                        SHOW_MULTI_ACTIONBAR_3 = 1
                    else
                        SHOW_MULTI_ACTIONBAR_3 = nil
                    end
                    
                    InterfaceOptions_UpdateMultiActionBars()
                    MultiActionBar_Update()
                end,
        },
        right1BarMouseover = {
            type = "toggle",
            name = L["Show On Mouseover"],
            desc = L["Show the bar only when the mouse is over it."],
            order = 42,
            get=function() return db.right1BarMouseover end,
            set=function()
                    db.right1BarMouseover = not db.right1BarMouseover
                end,
        },
        right1BarScale = {
            name = L["Bar Scale"],
            desc = L["Size of the bar."],
            type = "range",
            min = 0.1,
            max = 3,
            step = 0.05,
            order = 43,
            get=function() return db.right1BarScale end,
            set=function(self, val)
                    db.right1BarScale = val
                end,
        },
        
        header5 = {
            name = L["Right Bar 2"],
            type = "header",
            order = 50,
        },
        right2Bar = {
            type = "toggle",
            name = L["Show"],
            desc = L["Toggle the bar."],
            order = 51,
            get=function() return db.right2Bar end,
            set=function()
                    db.right2Bar = not db.right2Bar
                    
                    if db.right2Bar then
                        SHOW_MULTI_ACTIONBAR_4 = 1
                    else
                        SHOW_MULTI_ACTIONBAR_4 = nil
                    end
                    
                    InterfaceOptions_UpdateMultiActionBars()
                    MultiActionBar_Update()
                end,
        },
        
        header6 = {
            name = L["Pet Bar"],
            type = "header",
            order = 60,
        },
        petBar = {
            type = "toggle",
            name = L["Show"],
            desc = L["Toggle the bar."],
            order = 61,
            get=function() return db.petBar end,
            set=function()
                    db.petBar = not db.petBar
                end,
        },
        petBarMouseover = {
            type = "toggle",
            name = L["Show On Mouseover"],
            desc = L["Show the bar only when the mouse is over it."],
            order = 62,
            get=function() return db.petBarMouseover end,
            set=function()
                    db.petBarMouseover = not db.petBarMouseover
                end,
        },
        
        header7 = {
            name = L["Shapeshift Bar"],
            type = "header",
            order = 70,
        },
        shapeshiftBar = {
            type = "toggle",
            name = L["Show"],
            desc = L["Toggle the bar."],
            order = 71,
            get=function() return db.shapeshiftBar end,
            set=function()
                    db.shapeshiftBar = not db.shapeshiftBar
                end,
        },
        shapeshiftBarMouseover = {
            type = "toggle",
            name = L["Show On Mouseover"],
            desc = L["Show the bar only when the mouse is over it."],
            order = 72,
            get=function() return db.shapeshiftBarMouseover end,
            set=function()
                    db.shapeshiftBarMouseover = not db.shapeshiftBarMouseover
                end,
        },
    }
}

local optionsBuffs = {
    name = L["Buffs"],
    desc = L["Buff options."],
    type = "group",
    order = 5,
    args = {
        description1 = {
            name = L["Here you can move certain buffs and debuffs. They normally appear in the standard buff and debuff frames, unless you make an entry."],
            type = "description",
            order = 1,
        },
        header1 = {
            name = L["Short Buffs On You"],
            type = "header",
            order = 10,
        },
        buffFilterPlayerShort = {
            type = "select",
            name = L["Delete Buff Name"],
            desc = L["Click a Buff name to delete it from the list."],
            style = "dropdown",
            order = 12,
            values = function()
                    return db.buffFilterPlayerShort
                end,
            get=function()  end,
            set=function(self, i)
                    if db.buffFilterPlayerShort[i]==L["Close"] then
                        return
                    else
                        tremove(db.buffFilterPlayerShort, i)
                    end
                end,
        },
        buffFilterPlayerShortName = {
            name = L["Add Buff Name"],
            desc = L["Input Buff name and press enter to add it to the list."],
            type = "input",
            order = 13,
            get=function()  end,
            set=function(self, val)
                    val = strlower(val)
                    tinsert(db.buffFilterPlayerShort, val)
                end,
        },
        
        header2 = {
            name = L["Long Buffs On You"],
            type = "header",
            order = 20,
        },
        buffFilterPlayerLong = {
            type = "select",
            name = L["Delete Buff Name"],
            desc = L["Click a Buff name to delete it from the list."],
            style = "dropdown",
            order = 22,
            values = function()
                    return db.buffFilterPlayerLong
                end,
            get=function()  end,
            set=function(self, i)
                    if db.buffFilterPlayerLong[i]==L["Close"] then
                        return
                    else
                        tremove(db.buffFilterPlayerLong, i)
                    end
                end,
        },
        buffFilterPlayerLongName = {
            name = L["Add Buff Name"],
            desc = L["Input Buff name and press enter to add it to the list."],
            type = "input",
            order = 23,
            get=function()  end,
            set=function(self, val)
                    val = strlower(val)
                    tinsert(db.buffFilterPlayerLong, val)
                end,
        },
        
        header3 = {
            name = L["Raid Buffs On You"],
            type = "header",
            order = 30,
        },
        buffFilterPlayerRaidBuffs = {
            type = "select",
            name = L["Delete Buff Name"],
            desc = L["Click a Buff name to delete it from the list."],
            style = "dropdown",
            order = 32,
            values = function()
                    return db.buffFilterPlayerRaidBuffs
                end,
            get=function()  end,
            set=function(self, i)
                    if db.buffFilterPlayerRaidBuffs[i]==L["Close"] then
                        return
                    else
                        tremove(db.buffFilterPlayerRaidBuffs, i)
                    end
                end,
        },
        buffFilterPlayerRaidBuffsName = {
            name = L["Add Buff Name"],
            desc = L["Input Buff name and press enter to add it to the list."],
            type = "input",
            order = 33,
            get=function()  end,
            set=function(self, val)
                    val = strlower(val)
                    tinsert(db.buffFilterPlayerRaidBuffs, val)
                end,
        },
        
        header4 = {
            name = L["Don't Show These Debuffs On You"],
            type = "header",
            order = 40,
        },
        buffFilterPlayerDebuffs = {
            type = "select",
            name = L["Delete Buff Name"],
            desc = L["Click a Buff name to delete it from the list."],
            style = "dropdown",
            order = 42,
            values = function()
                    return db.buffFilterPlayerDebuffs
                end,
            get=function()  end,
            set=function(self, i)
                    if db.buffFilterPlayerDebuffs[i]==L["Close"] then
                        return
                    else
                        tremove(db.buffFilterPlayerDebuffs, i)
                    end
                end,
        },
        buffFilterPlayerDebuffsName = {
            name = L["Add Buff Name"],
            desc = L["Input Buff name and press enter to add it to the list."],
            type = "input",
            order = 43,
            get=function()  end,
            set=function(self, val)
                    val = strlower(val)
                    tinsert(db.buffFilterPlayerDebuffs, val)
                end,
        },
        
        header5 = {
            name = L["Debuffs On Target From You"],
            type = "header",
            order = 50,
        },
        buffFilterTargetMyDebuffs = {
            type = "select",
            name = L["Delete Buff Name"],
            desc = L["Click a Buff name to delete it from the list."],
            style = "dropdown",
            order = 52,
            values = function()
                    return db.buffFilterTargetMyDebuffs
                end,
            get=function()  end,
            set=function(self, i)
                    if db.buffFilterTargetMyDebuffs[i]==L["Close"] then
                        return
                    else
                        tremove(db.buffFilterTargetMyDebuffs, i)
                    end
                end,
        },
        buffFilterTargetMyDebuffsName = {
            name = L["Add Buff Name"],
            desc = L["Input Buff name and press enter to add it to the list."],
            type = "input",
            order = 53,
            get=function()  end,
            set=function(self, val)
                    val = strlower(val)
                    tinsert(db.buffFilterTargetMyDebuffs, val)
                end,
        },
        
        header6 = {
            name = L["Buffs On Target From You"],
            type = "header",
            order = 60,
        },
        buffFilterTargetMyBuffs = {
            type = "select",
            name = L["Delete Buff Name"],
            desc = L["Click a Buff name to delete it from the list."],
            style = "dropdown",
            order = 62,
            values = function()
                    return db.buffFilterTargetMyBuffs
                end,
            get=function()  end,
            set=function(self, i)
                    if db.buffFilterTargetMyBuffs[i]==L["Close"] then
                        return
                    else
                        tremove(db.buffFilterTargetMyBuffs, i)
                    end
                end,
        },
        buffFilterTargetMyBuffsName = {
            name = L["Add Buff Name"],
            desc = L["Input Buff name and press enter to add it to the list."],
            type = "input",
            order = 63,
            get=function()  end,
            set=function(self, val)
                    val = strlower(val)
                    tinsert(db.buffFilterTargetMyBuffs, val)
                end,
        },
        
        header7 = {
            name = L["Buffs On Target"],
            type = "header",
            order = 70,
        },
        buffFilterTargetOthersBuffs = {
            type = "select",
            name = L["Delete Buff Name"],
            desc = L["Click a Buff name to delete it from the list."],
            style = "dropdown",
            order = 72,
            values = function()
                    return db.buffFilterTargetOthersBuffs
                end,
            get=function()  end,
            set=function(self, i)
                    if db.buffFilterTargetOthersBuffs[i]==L["Close"] then
                        return
                    else
                        tremove(db.buffFilterTargetOthersBuffs, i)
                    end
                end,
        },
        buffFilterTargetOthersBuffsName = {
            name = L["Add Buff Name"],
            desc = L["Input Buff name and press enter to add it to the list."],
            type = "input",
            order = 73,
            get=function()  end,
            set=function(self, val)
                    val = strlower(val)
                    tinsert(db.buffFilterTargetOthersBuffs, val)
                end,
        },
        
        header9 = {
            name = L["Debuffs On Target"],
            type = "header",
            order = 75,
        },
        buffFilterTargetOthersDebuffs = {
            type = "select",
            name = L["Delete Buff Name"],
            desc = L["Click a Buff name to delete it from the list."],
            style = "dropdown",
            order = 76,
            values = function()
                    return db.buffFilterTargetOthersDebuffs
                end,
            get=function()  end,
            set=function(self, i)
                    if db.buffFilterTargetOthersDebuffs[i]==L["Close"] then
                        return
                    else
                        tremove(db.buffFilterTargetOthersDebuffs, i)
                    end
                end,
        },
        buffFilterTargetOthersDebuffsName = {
            name = L["Add Buff Name"],
            desc = L["Input Buff name and press enter to add it to the list."],
            type = "input",
            order = 77,
            get=function()  end,
            set=function(self, val)
                    val = strlower(val)
                    tinsert(db.buffFilterTargetOthersDebuffs, val)
                end,
        },
        
        header8 = {
            name = L["Buffs On Pet"],
            type = "header",
            order = 80,
        },
        buffFilterPet = {
            type = "select",
            name = L["Delete Buff Name"],
            desc = L["Click a Buff name to delete it from the list."],
            style = "dropdown",
            order = 82,
            values = function()
                    return db.buffFilterPet
                end,
            get=function()  end,
            set=function(self, i)
                    if db.buffFilterPet[i]==L["Close"] then
                        return
                    else
                        tremove(db.buffFilterPet, i)
                    end
                end,
        },
        buffFilterPetName = {
            name = L["Add Buff Name"],
            desc = L["Input Buff name and press enter to add it to the list."],
            type = "input",
            order = 83,
            get=function()  end,
            set=function(self, val)
                    val = strlower(val)
                    tinsert(db.buffFilterPet, val)
                end,
        },
        
    }
}

local optionsMisc = {
    name = L["Miscellaneous"],
    desc = L["Miscellaneous options."],
    type = "group",
    order = 10,
    args = {
        lootframe = {
            type="execute",
            name=L["Move Loot Window"],
            desc=L["Shows the anchor of the loot window. Click again to deactivate."],
            func=function(self) SlashCmdList["TEKSLOOT"]() end,
            order=0,
        },
        
        header0 = {
            name = L["Tooltip"],
            type = "header",
            order = 1,
        },
        tooltipAnchor = {
            type = "select",
            name = L["Anchor"],
            desc = L["Anchor of your tooltip."],
            style = "dropdown",
            order = 2,
            values = function()
                    return db.tooltipAnchorList
                end,
            get=function() return db.tooltipAnchor end,
            set=function(self, i)
                    if db.tooltipAnchorList[i]==L["Close"] then
                        return
                    else
                        db.tooltipAnchor = db.tooltipAnchorList[i]
                    end
                end,
        },
        dontShowTargetTooltip = {
            type = "toggle",
            name = L["Don't Show Target"],
            desc = L["Don't show the tooltip of your current target."],
            order = 3,
            get=function() return db.dontShowTargetTooltip end,
            set=function()
                    db.dontShowTargetTooltip = not db.dontShowTargetTooltip
                end,
        },
        
        header1 = {
            name = L["Errorspam"],
            type = "header",
            order = 10,
        },
        description1 = {
            name = L["errorHiderDescription"],
            type = "description",
            order = 11,
        },
        errorFilterList = {
            type = "select",
            name = L["Delete Error Text"],
            desc = L["Click an error text to delete it from the list."],
            style = "dropdown",
            width = "double",
            order = 12,
            values = function()
                    return db.errorFilterList
                end,
            get=function()  end,
            set=function(self, i)
                    if db.errorFilterList[i]==L["Close"] then
                        return
                    else
                        tremove(db.errorFilterList, i)
                    end
                end,
        },
        errorFilterName = {
            name = L["Add Error Text"],
            desc = L["Input an error text and press enter to add it to the list. Every error containing that text won't display."],
            type = "input",
            width = "double",
            order = 13,
            get=function()  end,
            set=function(self, val)
                    val = strlower(val)
                    tinsert(db.errorFilterList, val)
                end,
        },
        
        header2 = {
            name = L["Automation"],
            type = "header",
            order = 20,
        },
        declineDuels = {
            type = "toggle",
            name = L["Decline Duels"],
            desc = L["Automatically decline duels."],
            order = 23,
            get=function() return db.declineDuels end,
            set=function()
                    db.declineDuels = not db.declineDuels
                end,
        },
        presenceWarner = {
            type = "toggle",
            name = L["Presence Warner"],
            desc = L["Warning when you have the wrong presence selected."],
            order = 26,
            get=function() return db.presenceWarner end,
            set=function()
                    db.presenceWarner = not db.presenceWarner
                end,
        },
    }
}

-- Spawn the options
function DocsUI:SpawnOptions()
    db = DocsUI.db.profile
    
    DocsUI.options = {
        type = "group",
        handler = DocsUI,
        args = {
            docsUI = {
                name = "DocsUI",
                desc = "DocsUI",
                type = "group",
                order = 0,
                args = {
                    i = {
                        type="description",
                        image=db.logo,
                        imageWidth = 256,
                        imageHeight = 64,
                        imageCoords = {0,1,0,0.25},
                        name="",
                        order=0,
                    },
                    d = {
                        type="description",
                        name=L["rl"],
                        order=1,
                    },
                    reloadui = {
                        type="execute",
                        name=L["Reload Interface"],
                        desc=L["Reloads the interface to make your changes visible."],
                        func=function(self) ReloadUI() end,
                        order=2,
                    },
                    header0 = {
                        name = "",
                        type = "header",
                        order = 8,
                    },
                    reImport = {
                        type="execute",
                        name=L["Import settings"],
                        desc=L["Imports the standard settings again."],
                        func=function(self) DocsUI.importWindow:Show() end,
                        order=9,
                    },
                },
            },
            optionsGeneral = optionsGeneral,
            optionsUnitFrames = optionsUnitFrames,
            optionsBuffs = optionsBuffs,
            optionsActionbars = optionsActionbars,
            optionsMisc = optionsMisc,
        },
    }
end

